@@ -2,7 +2,7 @@
 ** dcalc
 ** The Sleuth Kit 
 **
-** $Date: 2005/03/25 03:12:34 $
+** $Date: 2005/06/15 15:58:02 $
 **
 ** Calculates the corresponding block number between 'dls' and 'dd' images
 ** when given an 'dls' block number, it determines the block number it
@@ -106,8 +106,8 @@ count_dls_act(FS_INFO * fs, DADDR_T addr, char *buf, int flags, void *ptr)
 static OFF_T flen;
 
 static uint8_t
-count_slack_file_act(FS_INFO * fs, DADDR_T addr, char *buf, int size,
-		     int flags, void *ptr)
+count_slack_file_act(FS_INFO * fs, DADDR_T addr, char *buf,
+		     unsigned int size, int flags, void *ptr)
 {
 
     if (verbose)
@@ -143,14 +143,14 @@ count_slack_file_act(FS_INFO * fs, DADDR_T addr, char *buf, int size,
 }
 
 static uint8_t
-count_slack_inode_act(FS_INFO * fs, INUM_T inum, FS_INODE * fs_inode,
+count_slack_inode_act(FS_INFO * fs, FS_INODE * fs_inode,
 		      int flags, void *ptr)
 {
 
     if (verbose)
 	fprintf(logfp,
 		"count_slack_inode_act: Processing meta data: %" PRIuINUM
-		"\n", inum);
+		"\n", fs_inode->addr);
 
     /* We will now do a file walk on the content */
     if ((fs->ftype & FSMASK) != NTFS_TYPE) {
