@@ -1,7 +1,7 @@
 /*
 ** The Sleuth Kit
 **
-** $Date: 2005/03/25 03:12:34 $
+** $Date: 2005/07/08 17:20:10 $
 **
 ** Brian Carrier [carrier@sleuthkit.org]
 ** Copyright (c) 2003-2005 Brian Carrier.  All rights reserved 
@@ -105,8 +105,8 @@ print_block(FS_INFO * fs, DADDR_T addr, char *buf, int flags, void *ptr)
 static OFF_T flen;
 
 static uint8_t
-slack_file_act(FS_INFO * fs, DADDR_T addr, char *buf, int size,
-	       int flags, void *ptr)
+slack_file_act(FS_INFO * fs, DADDR_T addr, char *buf,
+	       unsigned int size, int flags, void *ptr)
 {
 
     if (verbose)
@@ -135,14 +135,13 @@ slack_file_act(FS_INFO * fs, DADDR_T addr, char *buf, int size,
 
 /* Call back for inode_walk */
 static uint8_t
-slack_inode_act(FS_INFO * fs, INUM_T inum, FS_INODE * fs_inode, int flags,
-		void *ptr)
+slack_inode_act(FS_INFO * fs, FS_INODE * fs_inode, int flags, void *ptr)
 {
 
     if (verbose)
 	fprintf(logfp,
 		"slack_inode_act: Processing meta data: %" PRIuINUM "\n",
-		inum);
+		fs_inode->addr);
 
     /* We will now do a file walk on the content and print the
      * data after the specified size of the file */
