@@ -2,7 +2,7 @@
 ** dstat
 ** The Sleuth Kit 
 **
-** $Date: 2005/03/25 03:12:34 $
+** $Date: 2005/06/13 19:27:16 $
 **
 ** Get the details about a data unit
 **
@@ -73,12 +73,12 @@ dstat_act(FS_INFO * fs, DADDR_T addr, char *buf, int flags, void *ptr)
 
     if ((fs->ftype & FSMASK) == FFS_TYPE) {
 	FFS_INFO *ffs = (FFS_INFO *) fs;
-	printf("Group: %lu\n", (ULONG) ffs->cg_num);
+	printf("Group: %lu\n", (ULONG) ffs->grp_num);
     }
     else if ((fs->ftype & FSMASK) == EXTxFS_TYPE) {
 	EXT2FS_INFO *ext2fs = (EXT2FS_INFO *) fs;
 	if (addr >= ext2fs->first_data_block)
-	    printf("Group: %lu\n", (ULONG) ext2fs->grpnum);
+	    printf("Group: %lu\n", (ULONG) ext2fs->grp_num);
     }
     else if ((fs->ftype & FSMASK) == FATFS_TYPE) {
 	FATFS_INFO *fatfs = (FATFS_INFO *) fs;
@@ -99,6 +99,6 @@ dstat_act(FS_INFO * fs, DADDR_T addr, char *buf, int flags, void *ptr)
 uint8_t
 fs_dstat(FS_INFO * fs, uint8_t lclflags, DADDR_T addr, int flags)
 {
-    fs->block_walk(fs, addr, addr, flags, dstat_act, "dstat");
+    fs->block_walk(fs, addr, addr, flags, dstat_act, (void *) "dstat");
     return 0;
 }
