@@ -1,7 +1,7 @@
 /*
 ** The Sleuth Kit 
 **
-** $Date: 2005/03/25 03:12:34 $
+** $Date: 2005/07/08 17:20:10 $
 **
 ** Brian Carrier [carrier@sleuthkit.org]
 ** Copyright (c) 2003-2005 Brian Carrier.  All rights reserved 
@@ -28,39 +28,46 @@
 #include "ext2fs.h"
 
 
-/* ext2fs_group_lookup - look up group descriptor info */
-
+/* ext2fs_group_load - load block group descriptor into cache */
 static void
-ext2fs_group_lookup(EXT2FS_INFO * ext2fs, EXT2_GRPNUM_T grpnum)
+ext2fs_group_load(EXT2FS_INFO * ext2fs, EXT2_GRPNUM_T grp_num)
 {
-    ext2fs_gd *gd = ext2fs->group;
+    ext2fs_gd *gd;
     OFF_T offs;
 
     /*
      * Sanity check
      */
-    if (grpnum < 0 || grpnum >= ext2fs->groups_count)
-	error("invalid group descriptor number: %" PRI_EXT2GRP "", grpnum);
+    if (grp_num < 0 || grp_num >= ext2fs->groups_count)
+	error("invalid group descriptor number: %" PRI_EXT2GRP "",
+	      grp_num);
+
+    if (ext2fs->grp_buf == NULL)
+	ext2fs->grp_buf = (ext2fs_gd *) mymalloc(sizeof(ext2fs_gd));
+    else if (ext2fs->grp_num == grp_num)
+	return;
+
+    gd = ext2fs->grp_buf;
 
     /*
      * We're not reading group descriptors often, so it is OK to do small
      * reads instead of cacheing group descriptors in a large buffer.
      */
-    offs = ext2fs->group_offset + grpnum * sizeof(ext2fs_gd);
+    offs = ext2fs->groups_offset + grp_num * sizeof(ext2fs_gd);
     if (fs_read_random(&ext2fs->fs_info, (char *) gd,
 		       sizeof(ext2fs_gd), offs) != sizeof(ext2fs_gd)) {
-	error("ext2fs_group_lookup: Error reading group descriptor %d at %"
-	      PRIuOFF ": %m", grpnum, offs);
+	error("ext2fs_group_load: Error reading group descriptor %d at %"
+	      PRIuOFF ": %m", grp_num, offs);
     }
-    ext2fs->grpnum = grpnum;
+    ext2fs->grp_num = grp_num;
 
     if (verbose) {
 	FS_INFO *fs = (FS_INFO *) & ext2fs->fs_info;
 	fprintf(logfp,
 		"\tgroup %" PRI_EXT2GRP ": %" PRIu16 "/%" PRIu16
-		" free blocks/inodes\n", grpnum, getu16(fs,
-							gd->
-							bg_free_blocks_count),
+		" free blocks/inodes\n", grp_num, getu16(fs,
+							 gd->
+							 bg_free_blocks_count),
 		getu16(fs, gd->bg_free_inodes_count));
     }
 }
@@ -81,97 +88,104 @@ ext2fs_print_map(UCHAR * map, int len)
 }
 
 
-/* ext2fs_bmap_lookup - look up block bitmap */
-UCHAR *
-ext2fs_bmap_lookup(EXT2FS_INFO * ext2fs, EXT2_GRPNUM_T grpnum)
+/* ext2fs_bmap_load - look up block bitmap & load into cache */
+static void
+ext2fs_bmap_load(EXT2FS_INFO * ext2fs, EXT2_GRPNUM_T grp_num)
 {
     FS_INFO *fs = (FS_INFO *) & ext2fs->fs_info;
 
     /*
-     * Look up the group descriptor info.
+     * Look up the group descriptor info.  The load will do the sanity check.
      */
-    if (ext2fs->grpnum != grpnum)
-	ext2fs_group_lookup(ext2fs, grpnum);
+    if ((ext2fs->grp_buf == NULL) || (ext2fs->grp_num != grp_num))
+	ext2fs_group_load(ext2fs, grp_num);
+
+    if (ext2fs->bmap_buf == NULL)
+	ext2fs->bmap_buf = (unsigned char *) mymalloc(fs->block_size);
+    else if (ext2fs->bmap_grp_num == grp_num)
+	return;
 
     /*
      * Look up the block allocation bitmap.
      */
-
-    if (getu32(fs, ext2fs->group->bg_block_bitmap) > fs->last_block) {
-	error("ext2fs_bmap_lookup: Block too large for image: %" PRIu32 "",
-	      getu32(fs, ext2fs->group->bg_block_bitmap));
+    if (getu32(fs, ext2fs->grp_buf->bg_block_bitmap) > fs->last_block) {
+	error("ext2fs_bmap_load: Block too large for image: %" PRIu32 "",
+	      getu32(fs, ext2fs->grp_buf->bg_block_bitmap));
     }
 
-    if (fs_read_block_nobuf(fs, (char *) ext2fs->block_map,
+    if (fs_read_block_nobuf(fs, (char *) ext2fs->bmap_buf,
 			    ext2fs->fs_info.block_size,
 			    (DADDR_T) getu32(fs,
-					     ext2fs->group->
+					     ext2fs->grp_buf->
 					     bg_block_bitmap)) !=
 	ext2fs->fs_info.block_size) {
-	error("ext2fs_bmap_lookup: Error reading bitmap group %d at %"
-	      PRIu32 ": %m", grpnum, getu32(fs,
-					    ext2fs->group->
-					    bg_block_bitmap));
+	error("ext2fs_bmap_load: Error reading bitmap group %d at %"
+	      PRIu32 ": %m", grp_num, getu32(fs,
+					     ext2fs->grp_buf->
+					     bg_block_bitmap));
     }
 
-    ext2fs->bmap_num = grpnum;
+    ext2fs->bmap_grp_num = grp_num;
 
     if (verbose > 1)
-	ext2fs_print_map(ext2fs->block_map,
+	ext2fs_print_map(ext2fs->bmap_buf,
 			 getu32(fs, ext2fs->fs->s_blocks_per_group));
-
-    return (ext2fs->block_map);
 }
 
-/* ext2fs_imap_lookup - look up inode bitmap */
-
-UCHAR *
-ext2fs_imap_lookup(EXT2FS_INFO * ext2fs, EXT2_GRPNUM_T grpnum)
+/* ext2fs_imap_load - look up inode bitmap & load into cache */
+static void
+ext2fs_imap_load(EXT2FS_INFO * ext2fs, EXT2_GRPNUM_T grp_num)
 {
-
     FS_INFO *fs = (FS_INFO *) & ext2fs->fs_info;
     /*
      * Look up the group descriptor info.
      */
-    if (ext2fs->grpnum != grpnum)
-	ext2fs_group_lookup(ext2fs, grpnum);
+    if ((ext2fs->grp_buf == NULL) || (ext2fs->grp_num != grp_num))
+	ext2fs_group_load(ext2fs, grp_num);
+
+    /* Allocate the cache buffer and exit if map is already loaded */
+    if (ext2fs->imap_buf == NULL)
+	ext2fs->imap_buf = (UCHAR *) mymalloc(fs->block_size);
+    else if (ext2fs->imap_grp_num == grp_num)
+	return;
 
     /*
      * Look up the inode allocation bitmap.
      */
-    if (getu32(fs, ext2fs->group->bg_inode_bitmap) > fs->last_block) {
-	error("ext2fs_imap_lookup: Block too large for image: %" PRIu32 "",
-	      getu32(fs, ext2fs->group->bg_inode_bitmap));
+    if (getu32(fs, ext2fs->grp_buf->bg_inode_bitmap) > fs->last_block) {
+	error("ext2fs_imap_load: Block too large for image: %" PRIu32 "",
+	      getu32(fs, ext2fs->grp_buf->bg_inode_bitmap));
     }
+
     if (fs_read_block_nobuf(fs,
-			    (char *) ext2fs->inode_map,
+			    (char *) ext2fs->imap_buf,
 			    ext2fs->fs_info.block_size,
 			    (DADDR_T) getu32(fs,
-					     ext2fs->group->
+					     ext2fs->grp_buf->
 					     bg_inode_bitmap))
 	!= ext2fs->fs_info.block_size) {
 
-	error("ext2fs_imap_lookup: Error reading inode bitmap %d at %"
-	      PRIu32 ": %m", grpnum, getu32(fs,
-					    ext2fs->group->
-					    bg_inode_bitmap));
+	error("ext2fs_imap_load: Error reading inode bitmap %d at %"
+	      PRIu32 ": %m", grp_num, getu32(fs,
+					     ext2fs->grp_buf->
+					     bg_inode_bitmap));
     }
 
-    ext2fs->imap_num = grpnum;
+    ext2fs->imap_grp_num = grp_num;
     if (verbose > 1)
-	ext2fs_print_map(ext2fs->inode_map,
+	ext2fs_print_map(ext2fs->imap_buf,
 			 getu32(fs, ext2fs->fs->s_inodes_per_group));
 
-    return (ext2fs->inode_map);
+    return;
 }
 
-/* ext2fs_dinode_lookup - look up disk inode */
+/* ext2fs_dinode_load - look up disk inode & load into cache */
 
 static void
-ext2fs_dinode_lookup(EXT2FS_INFO * ext2fs, INUM_T inum)
+ext2fs_dinode_load(EXT2FS_INFO * ext2fs, INUM_T inum)
 {
-    ext2fs_inode *dino = ext2fs->dinode;
-    EXT2_GRPNUM_T grpnum;
+    ext2fs_inode *dino;
+    EXT2_GRPNUM_T grp_num;
     OFF_T addr;
     INUM_T rel_inum;
     FS_INFO *fs = (FS_INFO *) & ext2fs->fs_info;
@@ -182,31 +196,39 @@ ext2fs_dinode_lookup(EXT2FS_INFO * ext2fs, INUM_T inum)
     if ((inum < fs->first_inum) || (inum > fs->last_inum))
 	error("invalid inode number: %" PRIuINUM "", inum);
 
+    /* Allocate the buffer or return if already loaded */
+    if (ext2fs->dino_buf == NULL)
+	ext2fs->dino_buf = (ext2fs_inode *) mymalloc(ext2fs->inode_size);
+    else if (ext2fs->dino_inum == inum)
+	return;
+
+    dino = ext2fs->dino_buf;
+
     /*
      * Look up the group descriptor for this inode. 
      */
-    grpnum = (inum - fs->first_inum) /
+    grp_num = (inum - fs->first_inum) /
 	getu32(fs, ext2fs->fs->s_inodes_per_group);
 
-    if (ext2fs->grpnum != grpnum)
-	ext2fs_group_lookup(ext2fs, grpnum);
+    if ((ext2fs->grp_buf == NULL) || (ext2fs->grp_num != grp_num))
+	ext2fs_group_load(ext2fs, grp_num);
 
     /*
      * Look up the inode table block for this inode.
      */
     rel_inum =
-	(inum - 1) - getu32(fs, ext2fs->fs->s_inodes_per_group) * grpnum;
+	(inum - 1) - getu32(fs, ext2fs->fs->s_inodes_per_group) * grp_num;
     addr = (OFF_T) getu32(fs,
-			  ext2fs->group->bg_inode_table) *
+			  ext2fs->grp_buf->bg_inode_table) *
 	(OFF_T) fs->block_size + rel_inum * (OFF_T) ext2fs->inode_size;
 
     if (fs_read_random(fs, (char *) dino, ext2fs->inode_size,
 		       addr) != ext2fs->inode_size) {
-	error("ext2fs_dinode_lookup: Error reading inode %" PRIuINUM
+	error("ext2fs_dinode_load: Error reading inode %" PRIuINUM
 	      " from %" PRIuOFF "  %m", inum, addr);
     }
 
-    ext2fs->inum = inum;
+    ext2fs->dino_inum = inum;
     if (verbose)
 	fprintf(logfp,
 		"%" PRIuINUM " m/l/s=%o/%d/%" PRIuOFF
@@ -229,24 +251,27 @@ ext2fs_dinode_lookup(EXT2FS_INFO * ext2fs, INUM_T inum)
 		(ULONG) getu32(fs, dino->i_dtime));
 }
 
-/* ext2fs_copy_inode - copy disk inode to generic inode */
-
+/* ext2fs_dinode_copy - copy cached disk inode into generic inode */
 static void
-ext2fs_copy_inode(EXT2FS_INFO * ext2fs, FS_INODE * fs_inode)
+ext2fs_dinode_copy(EXT2FS_INFO * ext2fs, FS_INODE * fs_inode)
 {
     int i;
-    ext2fs_inode *in = ext2fs->dinode;
+    ext2fs_inode *in = ext2fs->dino_buf;
     FS_INFO *fs = (FS_INFO *) & ext2fs->fs_info;
     ext2fs_sb *sb = ext2fs->fs;
-    EXT2_GRPNUM_T grpnum;
-    UCHAR *imap = 0;
+    EXT2_GRPNUM_T grp_num;
     INUM_T ibase = 0;
 
+    if (in == NULL)
+	error("ext2fs_dinode_copy: Error -- dino_buf is NULL");
+
     fs_inode->mode = getu16(fs, in->i_mode);
     fs_inode->nlink = getu16(fs, in->i_nlink);
 
     fs_inode->size = getu32(fs, in->i_size);
 
+    fs_inode->addr = ext2fs->dino_inum;
+
     /* the general size value in the inode is only 32-bits,
      * but the i_dir_acl value is used for regular files to 
      * hold the upper 32-bits 
@@ -294,12 +319,15 @@ ext2fs_copy_inode(EXT2FS_INFO * ext2fs, FS_INODE * fs_inode)
      */
     if (((fs_inode->mode & FS_INODE_FMT) == FS_INODE_LNK) &&
 	(fs_inode->size < EXT2FS_MAXPATHLEN) && (fs_inode->size >= 0)) {
-	int count = 0, j;
+	unsigned int count = 0;
+
 
 	fs_inode->link = mymalloc(fs_inode->size + 1);
 
 	/* it is located directly in the pointers */
 	if (fs_inode->size < 4 * (EXT2FS_NDADDR + EXT2FS_NIADDR)) {
+	    unsigned int j;
+
 	    for (i = 0; i < (EXT2FS_NDADDR + EXT2FS_NIADDR) &&
 		 count < fs_inode->size; i++) {
 		char *ptr = (char *) &in->i_block[i];
@@ -334,7 +362,7 @@ ext2fs_copy_inode(EXT2FS_INFO * ext2fs, FS_INODE * fs_inode)
 				  fs_inode->direct_addr[i]) !=
 		    fs->block_size) {
 		    error
-			("ext2fs_copy_inode: Error reading symlink destination from %"
+			("ext2fs_dinode_copy: Error reading symlink destination from %"
 			 PRIuDADDR ": %m", fs_inode->direct_addr[i]);
 		}
 
@@ -352,17 +380,19 @@ ext2fs_copy_inode(EXT2FS_INFO * ext2fs, FS_INODE * fs_inode)
     }
 
     /* Fill in the flags value */
-    grpnum = (ext2fs->inum - fs->first_inum) /
+    grp_num = (ext2fs->dino_inum - fs->first_inum) /
 	getu32(fs, ext2fs->fs->s_inodes_per_group);
 
-    imap = ext2fs_imap_lookup(ext2fs, grpnum);
-    ibase = grpnum * getu32(fs, ext2fs->fs->s_inodes_per_group) +
+    if (ext2fs->imap_grp_num != grp_num)
+	ext2fs_imap_load(ext2fs, grp_num);
+
+    ibase = grp_num * getu32(fs, ext2fs->fs->s_inodes_per_group) +
 	fs->first_inum;
 
     /*
      * Apply the allocated/unallocated restriction.
      */
-    fs_inode->flags = (isset(imap, ext2fs->inum - ibase) ?
+    fs_inode->flags = (isset(ext2fs->imap_buf, ext2fs->dino_inum - ibase) ?
 		       FS_FLAG_META_ALLOC : FS_FLAG_META_UNALLOC);
 
     fs_inode->flags |= (fs_inode->nlink ?
@@ -371,7 +401,6 @@ ext2fs_copy_inode(EXT2FS_INFO * ext2fs, FS_INODE * fs_inode)
     /*
      * Apply the used/unused restriction.
      */
-
     fs_inode->flags |= (fs_inode->ctime ?
 			FS_FLAG_META_USED : FS_FLAG_META_UNUSED);
 
@@ -385,8 +414,8 @@ ext2fs_inode_lookup(FS_INFO * fs, INUM_T inum)
     EXT2FS_INFO *ext2fs = (EXT2FS_INFO *) fs;
     FS_INODE *fs_inode = fs_inode_alloc(EXT2FS_NDADDR, EXT2FS_NIADDR);
 
-    ext2fs_dinode_lookup(ext2fs, inum);
-    ext2fs_copy_inode(ext2fs, fs_inode);
+    ext2fs_dinode_load(ext2fs, inum);
+    ext2fs_dinode_copy(ext2fs, fs_inode);
 
 
     return (fs_inode);
@@ -405,8 +434,7 @@ ext2fs_inode_walk(FS_INFO * fs, INUM_T start_inum, INUM_T end_inum,
 {
     char *myname = "extXfs_inode_walk";
     EXT2FS_INFO *ext2fs = (EXT2FS_INFO *) fs;
-    EXT2_GRPNUM_T grpnum;
-    UCHAR *imap = 0;
+    EXT2_GRPNUM_T grp_num;
     INUM_T inum;
     INUM_T ibase = 0;
     FS_INODE *fs_inode = fs_inode_alloc(EXT2FS_NDADDR, EXT2FS_NIADDR);
@@ -433,27 +461,34 @@ ext2fs_inode_walk(FS_INFO * fs, INUM_T start_inum, INUM_T end_inum,
 	 * Be sure to use the proper group descriptor data. XXX Linux inodes
 	 * start at 1, as in Fortran.
 	 */
-	grpnum = (inum - 1) / getu32(fs, ext2fs->fs->s_inodes_per_group);
-	if (imap == 0 || ext2fs->imap_num != grpnum) {
-	    imap = ext2fs_imap_lookup(ext2fs, grpnum);
+	grp_num = (inum - 1) / getu32(fs, ext2fs->fs->s_inodes_per_group);
+	if ((ext2fs->imap_buf == NULL)
+	    || (ext2fs->imap_grp_num != grp_num)) {
+	    ext2fs_imap_load(ext2fs, grp_num);
 	    ibase =
-		grpnum * getu32(fs, ext2fs->fs->s_inodes_per_group) + 1;
+		grp_num * getu32(fs, ext2fs->fs->s_inodes_per_group) + 1;
+	}
+
+	/* In case we didn't need to load it the bitmap */
+	else if (inum == start_inum) {
+	    ibase =
+		grp_num * getu32(fs, ext2fs->fs->s_inodes_per_group) + 1;
 	}
 
 	/*
 	 * Apply the allocated/unallocated restriction.
 	 */
-	myflags = (isset(imap, inum - ibase) ?
+	myflags = (isset(ext2fs->imap_buf, inum - ibase) ?
 		   FS_FLAG_META_ALLOC : FS_FLAG_META_UNALLOC);
 	if ((flags & myflags) != myflags)
 	    continue;
 
-	ext2fs_dinode_lookup(ext2fs, inum);
+	ext2fs_dinode_load(ext2fs, inum);
 
 	/*
 	 * Apply the linked/unlinked restriction.
 	 */
-	myflags |= (getu16(fs, ext2fs->dinode->i_nlink) ?
+	myflags |= (getu16(fs, ext2fs->dino_buf->i_nlink) ?
 		    FS_FLAG_META_LINK : FS_FLAG_META_UNLINK);
 
 	if ((flags & myflags) != myflags)
@@ -463,7 +498,7 @@ ext2fs_inode_walk(FS_INFO * fs, INUM_T start_inum, INUM_T end_inum,
 	 * Apply the used/unused restriction.
 	 */
 
-	myflags |= (getu32(fs, ext2fs->dinode->i_ctime) ?
+	myflags |= (getu32(fs, ext2fs->dino_buf->i_ctime) ?
 		    FS_FLAG_META_USED : FS_FLAG_META_UNUSED);
 
 	if ((flags & myflags) != myflags)
@@ -473,10 +508,10 @@ ext2fs_inode_walk(FS_INFO * fs, INUM_T start_inum, INUM_T end_inum,
 	 * Fill in a file system-independent inode structure and pass control
 	 * to the application.
 	 */
-	ext2fs_copy_inode(ext2fs, fs_inode);
+	ext2fs_dinode_copy(ext2fs, fs_inode);
 	fs_inode->flags = myflags;
 
-	if (WALK_STOP == action(fs, inum, fs_inode, myflags, ptr)) {
+	if (WALK_STOP == action(fs, fs_inode, myflags, ptr)) {
 	    fs_inode_free(fs_inode);
 	    return;
 	}
@@ -501,8 +536,7 @@ ext2fs_block_walk(FS_INFO * fs, DADDR_T start_blk, DADDR_T end_blk,
     char *myname = "extXfs_block_walk";
     EXT2FS_INFO *ext2fs = (EXT2FS_INFO *) fs;
     DATA_BUF *data_buf = data_buf_alloc(fs->block_size);
-    EXT2_GRPNUM_T grpnum;
-    UCHAR *bmap = 0;
+    EXT2_GRPNUM_T grp_num;
     DADDR_T addr;
     DADDR_T dbase = 0;		/* first block number in group */
     DADDR_T dmin = 0;		/* first block after inodes */
@@ -557,13 +591,15 @@ ext2fs_block_walk(FS_INFO * fs, DADDR_T start_blk, DADDR_T end_blk,
 	    }
 	    continue;
 	}
-	grpnum = ext2_dtog_lcl(fs, ext2fs->fs, addr);
+
+	grp_num = ext2_dtog_lcl(fs, ext2fs->fs, addr);
 
 	/* Lookup bitmap if not loaded */
-	if (bmap == 0 || ext2fs->bmap_num != grpnum) {
-	    bmap = ext2fs_bmap_lookup(ext2fs, grpnum);
-	    dbase = ext2_cgbase_lcl(fs, ext2fs->fs, grpnum);
-	    dmin = getu32(fs, ext2fs->group->bg_inode_table) +
+	if ((ext2fs->bmap_buf == NULL)
+	    || (ext2fs->bmap_grp_num != grp_num)) {
+	    ext2fs_bmap_load(ext2fs, grp_num);
+	    dbase = ext2_cgbase_lcl(fs, ext2fs->fs, grp_num);
+	    dmin = getu32(fs, ext2fs->grp_buf->bg_inode_table) +
 		INODE_TABLE_SIZE(ext2fs);
 
 	    if (verbose)
@@ -571,17 +607,24 @@ ext2fs_block_walk(FS_INFO * fs, DADDR_T start_blk, DADDR_T end_blk,
 			"block_walk: loading group %" PRI_EXT2GRP
 			" dbase %" PRIuDADDR " bmap +%" PRIuDADDR
 			" imap +%" PRIuDADDR " inos +%" PRIuDADDR "..%"
-			PRIuDADDR "\n", grpnum, dbase, (DADDR_T) getu32(fs,
-									ext2fs->
-									group->
-									bg_block_bitmap)
+			PRIuDADDR "\n", grp_num, dbase,
+			(DADDR_T) getu32(fs,
+					 ext2fs->grp_buf->bg_block_bitmap)
 			- dbase, (DADDR_T) getu32(fs,
-						  ext2fs->group->
+						  ext2fs->grp_buf->
 						  bg_inode_bitmap) - dbase,
 			(DADDR_T) getu32(fs,
-					 ext2fs->group->bg_inode_table) -
+					 ext2fs->grp_buf->bg_inode_table) -
 			dbase, dmin - 1 - dbase);
+
 	}
+	/* In case we didn't need to load the bmap */
+	else if ((addr == start_blk) || (addr == ext2fs->first_data_block)) {
+	    dbase = ext2_cgbase_lcl(fs, ext2fs->fs, grp_num);
+	    dmin = getu32(fs, ext2fs->grp_buf->bg_inode_table) +
+		INODE_TABLE_SIZE(ext2fs);
+	}
+
 
 	/*
 	 * Pass blocks of interest to the application. Identify meta blocks
@@ -600,14 +643,14 @@ ext2fs_block_walk(FS_INFO * fs, DADDR_T start_blk, DADDR_T end_blk,
 	 * locations of superblocks and group descriptor blocks are reserved.
 	 * They just happen to be reserved for something else :-)
 	 */
-	myflags = (isset(bmap, addr - dbase) ?
+	myflags = (isset(ext2fs->bmap_buf, addr - dbase) ?
 		   FS_FLAG_DATA_ALLOC : FS_FLAG_DATA_UNALLOC);
 
 	if ((addr >= dbase
-	     && addr < getu32(fs, ext2fs->group->bg_block_bitmap))
-	    || (addr == getu32(fs, ext2fs->group->bg_block_bitmap))
-	    || (addr == getu32(fs, ext2fs->group->bg_inode_bitmap))
-	    || (addr >= getu32(fs, ext2fs->group->bg_inode_table)
+	     && addr < getu32(fs, ext2fs->grp_buf->bg_block_bitmap))
+	    || (addr == getu32(fs, ext2fs->grp_buf->bg_block_bitmap))
+	    || (addr == getu32(fs, ext2fs->grp_buf->bg_inode_bitmap))
+	    || (addr >= getu32(fs, ext2fs->grp_buf->bg_inode_table)
 		&& addr < dmin))
 	    myflags |= FS_FLAG_DATA_META;
 	else
@@ -714,7 +757,7 @@ ext2fs_file_walk_indir(FS_INFO * fs, DATA_BUF * buf[], OFF_T length,
     char *myname = "extXfs_file_walk_indir";
     OFF_T todo_count = length;
     uint32_t *iaddr;
-    int n;
+    unsigned int n;
 
     if (verbose)
 	fprintf(logfp, "%s: level %d block %" PRIu32 "\n", myname, level,
@@ -823,6 +866,10 @@ ext2fs_file_walk(FS_INFO * fs, FS_INODE * inode, uint32_t type,
     OFF_T read_b;
 
 
+    if (verbose)
+	fprintf(logfp, "ext2fs_file_walk: Processing file %" PRIuINUM "\n",
+		inode->addr);
+
     /*
      * Initialize a buffer for each level of indirection that is supported by
      * this inode. The level 0 buffer is sized to the logical block size used
@@ -896,7 +943,7 @@ ext2fs_fscheck(FS_INFO * fs, FILE * hFile)
 static void
 ext2fs_fsstat(FS_INFO * fs, FILE * hFile)
 {
-    int i;
+    unsigned int i;
     EXT2FS_INFO *ext2fs = (EXT2FS_INFO *) fs;
     ext2fs_sb *sb = ext2fs->fs;
     int ibpg;
@@ -1067,6 +1114,7 @@ ext2fs_fsstat(FS_INFO * fs, FILE * hFile)
 		break;
 
 	    or_in = fsi->dtime;
+	    fs_inode_free(fsi);
 	}
 	fprintf(hFile, "\n");
     }
@@ -1106,7 +1154,7 @@ ext2fs_fsstat(FS_INFO * fs, FILE * hFile)
 	DADDR_T cg_base;
 	INUM_T inum;
 
-	ext2fs_group_lookup(ext2fs, i);
+	ext2fs_group_load(ext2fs, i);
 	fprintf(hFile, "\nGroup: %d:\n", i);
 
 	inum = fs->first_inum + gets32(fs, sb->s_inodes_per_group) * i;
@@ -1135,7 +1183,7 @@ ext2fs_fsstat(FS_INFO * fs, FILE * hFile)
 	 */
 	if (((getu32(fs, ext2fs->fs->s_feature_ro_compat) &
 	      EXT2FS_FEATURE_RO_COMPAT_SPARSE_SUPER) &&
-	     (cg_base != getu32(fs, ext2fs->group->bg_block_bitmap))) ||
+	     (cg_base != getu32(fs, ext2fs->grp_buf->bg_block_bitmap))) ||
 	    ((getu32(fs, ext2fs->fs->s_feature_ro_compat) &
 	      EXT2FS_FEATURE_RO_COMPAT_SPARSE_SUPER) == 0)) {
 
@@ -1165,19 +1213,19 @@ ext2fs_fsstat(FS_INFO * fs, FILE * hFile)
 
 	/* The block bitmap is a full block */
 	fprintf(hFile, "    Data bitmap: %" PRIu32 " - %" PRIu32 "\n",
-		getu32(fs, ext2fs->group->bg_block_bitmap),
-		getu32(fs, ext2fs->group->bg_block_bitmap));
+		getu32(fs, ext2fs->grp_buf->bg_block_bitmap),
+		getu32(fs, ext2fs->grp_buf->bg_block_bitmap));
 
 
 	/* The inode bitmap is a full block */
 	fprintf(hFile, "    Inode bitmap: %" PRIu32 " - %" PRIu32 "\n",
-		getu32(fs, ext2fs->group->bg_inode_bitmap),
-		getu32(fs, ext2fs->group->bg_inode_bitmap));
+		getu32(fs, ext2fs->grp_buf->bg_inode_bitmap),
+		getu32(fs, ext2fs->grp_buf->bg_inode_bitmap));
 
 
 	fprintf(hFile, "    Inode Table: %" PRIu32 " - %" PRIu32 "\n",
-		getu32(fs, ext2fs->group->bg_inode_table),
-		getu32(fs, ext2fs->group->bg_inode_table) + ibpg - 1);
+		getu32(fs, ext2fs->grp_buf->bg_inode_table),
+		getu32(fs, ext2fs->grp_buf->bg_inode_table) + ibpg - 1);
 
 
 	fprintf(hFile, "    Data Blocks: ");
@@ -1185,7 +1233,7 @@ ext2fs_fsstat(FS_INFO * fs, FILE * hFile)
 	/* If we are in a sparse group, display the other addresses */
 	if ((getu32(fs, ext2fs->fs->s_feature_ro_compat) &
 	     EXT2FS_FEATURE_RO_COMPAT_SPARSE_SUPER) &&
-	    (cg_base == getu32(fs, ext2fs->group->bg_block_bitmap))) {
+	    (cg_base == getu32(fs, ext2fs->grp_buf->bg_block_bitmap))) {
 
 	    /* it goes from the end of the inode bitmap to before the
 	     * table
@@ -1193,13 +1241,13 @@ ext2fs_fsstat(FS_INFO * fs, FILE * hFile)
 	     * This hard coded aspect does not scale ...
 	     */
 	    fprintf(hFile, "%" PRIu32 " - %" PRIu32 ", ",
-		    getu32(fs, ext2fs->group->bg_inode_bitmap) + 1,
-		    getu32(fs, ext2fs->group->bg_inode_table) - 1);
+		    getu32(fs, ext2fs->grp_buf->bg_inode_bitmap) + 1,
+		    getu32(fs, ext2fs->grp_buf->bg_inode_table) - 1);
 	}
 
 	fprintf(hFile, "%" PRIuDADDR " - %" PRIuDADDR "\n",
 		(uint64_t) getu32(fs,
-				  ext2fs->group->bg_inode_table) + ibpg,
+				  ext2fs->grp_buf->bg_inode_table) + ibpg,
 		((ext2_cgbase_lcl(fs, sb, i + 1) - 1) <
 		 fs->last_block) ? (ext2_cgbase_lcl(fs, sb,
 						    i + 1) -
@@ -1212,17 +1260,17 @@ ext2fs_fsstat(FS_INFO * fs, FILE * hFile)
 	/* The last group may not have a full number of blocks */
 	if (i != (ext2fs->groups_count - 1)) {
 	    fprintf(hFile, "  Free Inodes: %" PRIu16 " (%d%%)\n",
-		    getu16(fs, ext2fs->group->bg_free_inodes_count),
+		    getu16(fs, ext2fs->grp_buf->bg_free_inodes_count),
 		    (100 *
 		     getu16(fs,
-			    ext2fs->group->bg_free_inodes_count)) /
+			    ext2fs->grp_buf->bg_free_inodes_count)) /
 		    getu32(fs, sb->s_inodes_per_group));
 
 	    fprintf(hFile, "  Free Blocks: %" PRIu16 " (%d%%)\n",
-		    getu16(fs, ext2fs->group->bg_free_blocks_count),
+		    getu16(fs, ext2fs->grp_buf->bg_free_blocks_count),
 		    (100 *
 		     getu16(fs,
-			    ext2fs->group->bg_free_blocks_count)) /
+			    ext2fs->grp_buf->bg_free_blocks_count)) /
 		    getu32(fs, sb->s_blocks_per_group));
 	}
 	else {
@@ -1233,10 +1281,10 @@ ext2fs_fsstat(FS_INFO * fs, FILE * hFile)
 		num_left = getu32(fs, sb->s_inodes_per_group);
 
 	    fprintf(hFile, "  Free Inodes: %" PRIu16 " (%d%%)\n",
-		    getu16(fs, ext2fs->group->bg_free_inodes_count),
+		    getu16(fs, ext2fs->grp_buf->bg_free_inodes_count),
 		    (100 *
 		     getu16(fs,
-			    ext2fs->group->bg_free_inodes_count)) /
+			    ext2fs->grp_buf->bg_free_inodes_count)) /
 		    num_left);
 
 	    /* Now blocks */
@@ -1246,15 +1294,15 @@ ext2fs_fsstat(FS_INFO * fs, FILE * hFile)
 		num_left = getu32(fs, sb->s_blocks_per_group);
 
 	    fprintf(hFile, "  Free Blocks: %" PRIu16 " (%d%%)\n",
-		    getu16(fs, ext2fs->group->bg_free_blocks_count),
+		    getu16(fs, ext2fs->grp_buf->bg_free_blocks_count),
 		    (100 *
 		     getu16(fs,
-			    ext2fs->group->bg_free_blocks_count)) /
+			    ext2fs->grp_buf->bg_free_blocks_count)) /
 		    num_left);
 	}
 
 	fprintf(hFile, "  Total Directories: %" PRIu16 "\n",
-		getu16(fs, ext2fs->group->bg_used_dirs_count));
+		getu16(fs, ext2fs->grp_buf->bg_used_dirs_count));
     }
 
     return;
@@ -1306,7 +1354,7 @@ typedef struct {
 
 static uint8_t
 print_addr_act(FS_INFO * fs, DADDR_T addr, char *buf,
-	       int size, int flags, void *ptr)
+	       unsigned int size, int flags, void *ptr)
 {
     EXT2FS_PRINT_ADDR *print = (EXT2FS_PRINT_ADDR *) ptr;
 
@@ -1351,9 +1399,9 @@ ext2fs_istat(FS_INFO * fs, FILE * hFile, INUM_T inum, int numblock,
     fprintf(hFile, "%sAllocated\n",
 	    (fs_inode->flags & FS_FLAG_META_ALLOC) ? "" : "Not ");
 
-    fprintf(hFile, "Group: %lu\n", (ULONG) ext2fs->grpnum);
+    fprintf(hFile, "Group: %lu\n", (ULONG) ext2fs->grp_num);
     fprintf(hFile, "Generation Id: %" PRIu32 "\n",
-	    getu32(fs, ext2fs->dinode->i_generation));
+	    getu32(fs, ext2fs->dino_buf->i_generation));
 
     if (fs_inode->link)
 	fprintf(hFile, "symbolic link to: %s\n", fs_inode->link);
@@ -1368,34 +1416,34 @@ ext2fs_istat(FS_INFO * fs, FILE * hFile, INUM_T inum, int numblock,
     if (((fs_inode->mode & FS_INODE_FMT) == FS_INODE_BLK) ||
 	((fs_inode->mode & FS_INODE_FMT) == FS_INODE_CHR)) {
 	fprintf(hFile, "Device Major: %" PRIu8 "   Minor: %" PRIu8 "\n",
-		ext2fs->dinode->i_block[0][1],
-		ext2fs->dinode->i_block[0][0]);
+		ext2fs->dino_buf->i_block[0][1],
+		ext2fs->dino_buf->i_block[0][0]);
     }
 
-    if (getu32(fs, ext2fs->dinode->i_flags)) {
+    if (getu32(fs, ext2fs->dino_buf->i_flags)) {
 	fprintf(hFile, "Flags: ");
-	if (getu32(fs, ext2fs->dinode->i_flags) & EXT2_IN_SECDEL)
+	if (getu32(fs, ext2fs->dino_buf->i_flags) & EXT2_IN_SECDEL)
 	    fprintf(hFile, "Secure Delete, ");
 
-	if (getu32(fs, ext2fs->dinode->i_flags) & EXT2_IN_UNRM)
+	if (getu32(fs, ext2fs->dino_buf->i_flags) & EXT2_IN_UNRM)
 	    fprintf(hFile, "Undelete, ");
 
-	if (getu32(fs, ext2fs->dinode->i_flags) & EXT2_IN_COMP)
+	if (getu32(fs, ext2fs->dino_buf->i_flags) & EXT2_IN_COMP)
 	    fprintf(hFile, "Compressed, ");
 
-	if (getu32(fs, ext2fs->dinode->i_flags) & EXT2_IN_SYNC)
+	if (getu32(fs, ext2fs->dino_buf->i_flags) & EXT2_IN_SYNC)
 	    fprintf(hFile, "Sync Updates, ");
 
-	if (getu32(fs, ext2fs->dinode->i_flags) & EXT2_IN_IMM)
+	if (getu32(fs, ext2fs->dino_buf->i_flags) & EXT2_IN_IMM)
 	    fprintf(hFile, "Immutable, ");
 
-	if (getu32(fs, ext2fs->dinode->i_flags) & EXT2_IN_APPEND)
+	if (getu32(fs, ext2fs->dino_buf->i_flags) & EXT2_IN_APPEND)
 	    fprintf(hFile, "Append Only, ");
 
-	if (getu32(fs, ext2fs->dinode->i_flags) & EXT2_IN_NODUMP)
+	if (getu32(fs, ext2fs->dino_buf->i_flags) & EXT2_IN_NODUMP)
 	    fprintf(hFile, "Do Not Dump, ");
 
-	if (getu32(fs, ext2fs->dinode->i_flags) & EXT2_IN_NOA)
+	if (getu32(fs, ext2fs->dino_buf->i_flags) & EXT2_IN_NOA)
 	    fprintf(hFile, "No A-Time, ");
 
 	fprintf(hFile, "\n");
@@ -1409,16 +1457,16 @@ ext2fs_istat(FS_INFO * fs, FILE * hFile, INUM_T inum, int numblock,
      * value is stored at the end of the block.  There are 4 null bytes
      * in between the headers and values 
      */
-    if (getu32(fs, ext2fs->dinode->i_file_acl) != 0) {
+    if (getu32(fs, ext2fs->dino_buf->i_file_acl) != 0) {
 	char *buf = mymalloc(fs->block_size);
 	ext2fs_ea_header *ea_head;
 	ext2fs_ea_entry *ea_entry;
 
 	fprintf(hFile, "\nExtended Attributes  (Block: %" PRIu32 ")\n",
-		getu32(fs, ext2fs->dinode->i_file_acl));
+		getu32(fs, ext2fs->dino_buf->i_file_acl));
 
 	/* Is the value too big? */
-	if (getu32(fs, ext2fs->dinode->i_file_acl) > fs->last_block) {
+	if (getu32(fs, ext2fs->dino_buf->i_file_acl) > fs->last_block) {
 	    fprintf(hFile,
 		    "Extended Attributes block is larger than file system\n");
 	    goto egress_ea;
@@ -1426,11 +1474,11 @@ ext2fs_istat(FS_INFO * fs, FILE * hFile, INUM_T inum, int numblock,
 
 	if (fs_read_block_nobuf(fs, buf, fs->block_size,
 				(DADDR_T) getu32(fs,
-						 ext2fs->dinode->
+						 ext2fs->dino_buf->
 						 i_file_acl)) !=
 	    fs->block_size) {
 	    error("ext2fs_istat: Error reading ACL block %" PRIu32 " %m",
-		  getu32(fs, ext2fs->dinode->i_file_acl));
+		  getu32(fs, ext2fs->dino_buf->i_file_acl));
 	}
 
 
@@ -1476,7 +1524,7 @@ ext2fs_istat(FS_INFO * fs, FILE * hFile, INUM_T inum, int numblock,
 
 
 	    /* Copy the name into a buffer - not NULL term */
-	    strncpy(name, &ea_entry->name, ea_entry->nlen);
+	    strncpy(name, (char *) &ea_entry->name, ea_entry->nlen);
 	    name[ea_entry->nlen] = '\0';
 
 
@@ -1676,6 +1724,8 @@ ext2fs_istat(FS_INFO * fs, FILE * hFile, INUM_T inum, int numblock,
 	    fprintf(hFile, "\n");
     }
 
+    fs_inode_free(fs_inode);
+
     return;
 }
 
@@ -1688,10 +1738,18 @@ ext2fs_close(FS_INFO * fs)
     EXT2FS_INFO *ext2fs = (EXT2FS_INFO *) fs;
 
     free((char *) ext2fs->fs);
-    free((char *) ext2fs->dinode);
-    free((char *) ext2fs->group);
-    free((char *) ext2fs->block_map);
-    free((char *) ext2fs->inode_map);
+    if (ext2fs->dino_buf != NULL)
+	free((char *) ext2fs->dino_buf);
+
+    if (ext2fs->grp_buf != NULL)
+	free((char *) ext2fs->grp_buf);
+
+    if (ext2fs->bmap_buf != NULL)
+	free((char *) ext2fs->bmap_buf);
+
+    if (ext2fs->imap_buf != NULL)
+	free((char *) ext2fs->imap_buf);
+
     free(ext2fs);
 }
 
@@ -1701,7 +1759,7 @@ FS_INFO *
 ext2fs_open(IMG_INFO * img_info, unsigned char ftype, uint8_t test)
 {
     EXT2FS_INFO *ext2fs = (EXT2FS_INFO *) mymalloc(sizeof(*ext2fs));
-    int len;
+    unsigned int len;
     FS_INFO *fs = &(ext2fs->fs_info);
 
     if ((ftype & FSMASK) != EXTxFS_TYPE)
@@ -1813,7 +1871,7 @@ ext2fs_open(IMG_INFO * img_info, unsigned char ftype, uint8_t test)
      * super block
      */
 
-    ext2fs->group_offset =
+    ext2fs->groups_offset =
 	roundup((EXT2FS_SBOFF + sizeof(ext2fs_sb)), fs->block_size);
 
     ext2fs->groups_count = (getu32(fs, ext2fs->fs->s_blocks_count) -
@@ -1844,20 +1902,20 @@ ext2fs_open(IMG_INFO * img_info, unsigned char ftype, uint8_t test)
     /* allocate buffers */
 
     /* inode map */
-    ext2fs->inode_map = (unsigned char *) mymalloc(fs->block_size);
-    ext2fs->imap_num = -1;
+    ext2fs->imap_buf = NULL;
+    ext2fs->imap_grp_num = 0xffffffff;
 
     /* block map */
-    ext2fs->block_map = (unsigned char *) mymalloc(fs->block_size);
-    ext2fs->bmap_num = -1;
+    ext2fs->bmap_buf = NULL;
+    ext2fs->bmap_grp_num = 0xffffffff;
 
     /* dinode */
-    ext2fs->dinode = (ext2fs_inode *) mymalloc(ext2fs->inode_size);
-    ext2fs->inum = -1;
+    ext2fs->dino_buf = NULL;
+    ext2fs->dino_inum = 0xffffffff;
 
     /* group descriptor */
-    ext2fs->group = (ext2fs_gd *) mymalloc(sizeof(ext2fs_gd));
-    ext2fs->grpnum = -1;
+    ext2fs->grp_buf = NULL;
+    ext2fs->grp_num = 0xffffffff;
 
 
     /*
