@@ -1,7 +1,7 @@
 /*
 ** The Sleuth Kit 
 **
-** $Date: 2005/01/17 22:40:16 $
+** $Date: 2005/06/13 19:27:16 $
 **
 ** Brian Carrier [carrier@sleuthkit.org]
 ** Copyright (c) 2003-2005 Brian Carrier.  All rights reserved
@@ -387,16 +387,19 @@ typedef struct {
     FS_INFO fs_info;		/* super class */
     ext2fs_sb *fs;		/* super block */
 
-    ext2fs_gd *group;		/* cached group descriptor */
-    EXT2_GRPNUM_T grpnum;	/* cached group number */
-    UCHAR *block_map;		/* cached block allocation bitmap */
-    EXT2_GRPNUM_T bmap_num;	/* cached block bitmap nr */
-    UCHAR *inode_map;		/* cached inode allocation bitmap */
-    EXT2_GRPNUM_T imap_num;	/* cached inode bitmap nr */
-    ext2fs_inode *dinode;	/* cached disk inode */
-    INUM_T inum;		/* cached inode number */
+    ext2fs_gd *grp_buf;		/* cached group descriptor */
+    EXT2_GRPNUM_T grp_num;	/* cached group number */
 
-    OFF_T group_offset;		/* offset to first group desc */
+    UCHAR *bmap_buf;		/* cached block allocation bitmap */
+    EXT2_GRPNUM_T bmap_grp_num;	/* cached block bitmap nr */
+
+    UCHAR *imap_buf;		/* cached inode allocation bitmap */
+    EXT2_GRPNUM_T imap_grp_num;	/* cached inode bitmap nr */
+
+    ext2fs_inode *dino_buf;	/* cached disk inode */
+    INUM_T dino_inum;		/* cached inode number */
+
+    OFF_T groups_offset;	/* offset to first group desc */
     EXT2_GRPNUM_T groups_count;	/* nr of descriptor group blocks */
     uint8_t deentry_type;	/* v1 or v2 of dentry */
     uint16_t inode_size;	/* size of each inode */
