@@ -2,7 +2,7 @@
 ** ext2fs_dent
 ** The Sleuth Kit 
 **
-** $Date: 2005/03/25 01:55:23 $
+** $Date: 2005/06/14 04:10:01 $
 **
 ** Human Interface Layer Support for an EXT2FS image
 **
@@ -53,18 +53,30 @@
 #include "fs_tools.h"
 #include "ext2fs.h"
 
-
-/* Recursive path stuff */
-static unsigned int depth = 0;	/* how deep in the directory tree are we */
 #define MAX_DEPTH   64
-static char *didx[MAX_DEPTH];	/* pointer in dirs string to where '/' is for
-				 ** given depth */
 #define DIR_STRSZ  2048
-static char dirs[DIR_STRSZ];	/* The current directory name string */
 
+typedef struct {
+    /* Recursive path stuff */
+
+    /* how deep in the directory tree are we */
+    unsigned int depth;
+
+    /* pointer in dirs string to where '/' is for given depth */
+    char *didx[MAX_DEPTH];
+
+    /* The current directory name string */
+    char dirs[DIR_STRSZ];
+
+} EXT2FS_DINFO;
+
+
+static void ext2fs_dent_walk_lcl(FS_INFO *, EXT2FS_DINFO *, INUM_T, int,
+				 FS_DENT_WALK_FN, void *);
 
 static void
-ext2fs_dent_copy(EXT2FS_INFO * ext2fs, char *ext2_dent, FS_DENT * fs_dent)
+ext2fs_dent_copy(EXT2FS_INFO * ext2fs, EXT2FS_DINFO * dinfo,
+		 char *ext2_dent, FS_DENT * fs_dent)
 {
     int i;
     FS_INFO *fs = &(ext2fs->fs_info);
@@ -74,7 +86,6 @@ ext2fs_dent_copy(EXT2FS_INFO * ext2fs, char *ext2_dent, FS_DENT * fs_dent)
 
 	fs_dent->inode = getu32(fs, dir->inode);
 	fs_dent->namlen = getu16(fs, dir->name_len);
-	fs_dent->reclen = EXT2FS_DIRSIZ_lcl(getu16(fs, dir->name_len));
 
 	/* ext2 does not null terminate */
 	if (fs_dent->namlen >= fs_dent->maxnamlen)
@@ -93,7 +104,6 @@ ext2fs_dent_copy(EXT2FS_INFO * ext2fs, char *ext2_dent, FS_DENT * fs_dent)
 
 	fs_dent->inode = getu32(fs, dir->inode);
 	fs_dent->namlen = dir->name_len;
-	fs_dent->reclen = EXT2FS_DIRSIZ_lcl(dir->name_len);
 
 	/* ext2 does not null terminate */
 	if (fs_dent->namlen >= fs_dent->maxnamlen)
@@ -134,8 +144,8 @@ ext2fs_dent_copy(EXT2FS_INFO * ext2fs, char *ext2_dent, FS_DENT * fs_dent)
 
     }
 
-    fs_dent->path = dirs;
-    fs_dent->pathdepth = depth;
+    fs_dent->path = dinfo->dirs;
+    fs_dent->pathdepth = dinfo->depth;
 
     /* replace non-printable with ^. */
     for (i = 0; i < fs_dent->namlen; i++) {
@@ -168,7 +178,7 @@ ext2fs_dent_copy(EXT2FS_INFO * ext2fs, char *ext2_dent, FS_DENT * fs_dent)
 	/* Get inode */
 
 	if (fs_dent->fsi)
-	    free(fs_dent->fsi);
+	    fs_inode_free(fs_dent->fsi);
 
 	fs_dent->fsi = fs->inode_lookup(fs, fs_dent->inode);
 
@@ -190,12 +200,13 @@ ext2fs_dent_copy(EXT2FS_INFO * ext2fs, char *ext2_dent, FS_DENT * fs_dent)
 ** return how much was read this time or 0 if action wanted to stop
 */
 static int
-ext2fs_dent_parse_block(EXT2FS_INFO * ext2fs, char *buf, int len,
-			int flags, FS_DENT_WALK_FN action, void *ptr)
+ext2fs_dent_parse_block(EXT2FS_INFO * ext2fs, EXT2FS_DINFO * dinfo,
+			char *buf, int len, int flags,
+			FS_DENT_WALK_FN action, void *ptr)
 {
     FS_INFO *fs = &(ext2fs->fs_info);
 
-    int idellen = 0;
+    int dellen = 0;
     int idx;
     uint16_t reclen;
     uint32_t inode;
@@ -245,8 +256,16 @@ ext2fs_dent_parse_block(EXT2FS_INFO * ext2fs, char *buf, int len,
 
 	    /* we don't have a valid entry, skip ahead 4 */
 	    minreclen = 4;
-	    if (idellen > 0)
-		idellen -= 4;
+	    if (dellen > 0)
+		dellen -= 4;
+	    continue;
+	}
+
+	/* Before we process an entry in unallocated space, make
+	 * sure that it also ends in the unalloc space */
+	if ((dellen) && (dellen < minreclen)) {
+	    minreclen = 4;
+	    dellen -= 4;
 	    continue;
 	}
 
@@ -261,28 +280,34 @@ ext2fs_dent_parse_block(EXT2FS_INFO * ext2fs, char *buf, int len,
 
 	    int myflags = 0;
 
-	    ext2fs_dent_copy(ext2fs, dirPtr, fs_dent);
+	    ext2fs_dent_copy(ext2fs, dinfo, dirPtr, fs_dent);
 
 	    myflags = 0;
 	    /* Do we have a deleted entry? */
-	    if ((idellen > 0) || (inode == 0)) {
+	    if ((dellen > 0) || (inode == 0)) {
 		myflags |= FS_FLAG_NAME_UNALLOC;
-		idellen -= minreclen;
-		if (flags & FS_FLAG_NAME_UNALLOC)
-		    if (WALK_STOP == action(fs, fs_dent, myflags, ptr))
+		dellen -= minreclen;
+		if (flags & FS_FLAG_NAME_UNALLOC) {
+		    if (WALK_STOP == action(fs, fs_dent, myflags, ptr)) {
+			fs_dent_free(fs_dent);
 			return 0;
+		    }
+		}
 	    }
 	    /* We have a non-deleted entry */
 	    else {
 		myflags |= FS_FLAG_NAME_ALLOC;
-		if (flags & FS_FLAG_NAME_ALLOC)
-		    if (WALK_STOP == action(fs, fs_dent, myflags, ptr))
+		if (flags & FS_FLAG_NAME_ALLOC) {
+		    if (WALK_STOP == action(fs, fs_dent, myflags, ptr)) {
+			fs_dent_free(fs_dent);
 			return 0;
+		    }
+		}
 	    }
 
 	    /* If the actual length is shorter then the 
 	     ** recorded length, then the next entry(ies) have been 
-	     ** deleted.  Set idellen to the length of data that 
+	     ** deleted.  Set dellen to the length of data that 
 	     ** has been deleted
 	     **
 	     ** Because we aren't guaranteed with Ext2FS that the next
@@ -291,8 +316,8 @@ ext2fs_dent_parse_block(EXT2FS_INFO * ext2fs, char *buf, int len,
 	     ** before we waste time searching it
 	     */
 	    if ((reclen - minreclen >= EXT2FS_DIRSIZ_lcl(1))
-		&& (idellen <= 0))
-		idellen = reclen - minreclen;
+		&& (dellen <= 0))
+		dellen = reclen - minreclen;
 
 
 
@@ -302,19 +327,20 @@ ext2fs_dent_parse_block(EXT2FS_INFO * ext2fs, char *buf, int len,
 		(!ISDOT(fs_dent->name)) &&
 		((fs_dent->fsi->mode & FS_INODE_FMT) == FS_INODE_DIR)) {
 
-		if (depth < MAX_DEPTH) {
-		    didx[depth] = &dirs[strlen(dirs)];
-		    strncpy(didx[depth], fs_dent->name,
-			    DIR_STRSZ - strlen(dirs));
-		    strncat(dirs, "/", DIR_STRSZ);
+		if (dinfo->depth < MAX_DEPTH) {
+		    dinfo->didx[dinfo->depth] =
+			&dinfo->dirs[strlen(dinfo->dirs)];
+		    strncpy(dinfo->didx[dinfo->depth], fs_dent->name,
+			    DIR_STRSZ - strlen(dinfo->dirs));
+		    strncat(dinfo->dirs, "/", DIR_STRSZ);
 		}
-		depth++;
-		ext2fs_dent_walk(&(ext2fs->fs_info), fs_dent->inode,
-				 flags, action, ptr);
+		dinfo->depth++;
+		ext2fs_dent_walk_lcl(&(ext2fs->fs_info), dinfo,
+				     fs_dent->inode, flags, action, ptr);
 
-		depth--;
-		if (depth < MAX_DEPTH)
-		    *didx[depth] = '\0';
+		dinfo->depth--;
+		if (dinfo->depth < MAX_DEPTH)
+		    *dinfo->didx[dinfo->depth] = '\0';
 	    }
 
 	}			/* end of if for inode == 0 etc */
@@ -323,11 +349,15 @@ ext2fs_dent_parse_block(EXT2FS_INFO * ext2fs, char *buf, int len,
 	     ** increment to next word and try again
 	     */
 	    minreclen = 4;
+
+	    if (dellen > 0)
+		dellen -= 4;
 	}
 
-    }				/* end for */
-    return len;
+    }
 
+    fs_dent_free(fs_dent);
+    return len;
 }				/* end ext2fs_dent_parse_block() */
 
 
@@ -345,6 +375,16 @@ ext2fs_dent_parse_block(EXT2FS_INFO * ext2fs, char *buf, int len,
 void
 ext2fs_dent_walk(FS_INFO * fs, INUM_T inode, int flags,
 		 FS_DENT_WALK_FN action, void *ptr)
+{
+    EXT2FS_DINFO dinfo;
+
+    memset(&dinfo, 0, sizeof(EXT2FS_DINFO));
+    ext2fs_dent_walk_lcl(fs, &dinfo, inode, flags, action, ptr);
+}
+
+static void
+ext2fs_dent_walk_lcl(FS_INFO * fs, EXT2FS_DINFO * dinfo, INUM_T inode,
+		     int flags, FS_DENT_WALK_FN action, void *ptr)
 {
     FS_INODE *fs_inode;
     EXT2FS_INFO *ext2fs = (EXT2FS_INFO *) fs;
@@ -355,6 +395,11 @@ ext2fs_dent_walk(FS_INFO * fs, INUM_T inode, int flags,
     if (inode < fs->first_inum || inode > fs->last_inum)
 	error("invalid inode value: %i\n", inode);
 
+    if (verbose)
+	fprintf(logfp,
+		"ext2fs_dent_walk: Processing directory %" PRIuINUM "\n",
+		inode);
+
     fs_inode = fs->inode_lookup(fs, inode);
     size = roundup(fs_inode->size, fs->block_size);
     dirbuf = mymalloc(size);
@@ -372,8 +417,9 @@ ext2fs_dent_walk(FS_INFO * fs, INUM_T inode, int flags,
     while (size > 0) {
 	int len = (fs->block_size < size) ? fs->block_size : size;
 
-	int retval = ext2fs_dent_parse_block(ext2fs, dirptr, len, flags,
-					     action, ptr);
+	int retval =
+	    ext2fs_dent_parse_block(ext2fs, dinfo, dirptr, len, flags,
+				    action, ptr);
 
 	/* if 0, then the action wants to stop */
 	if (retval)
@@ -384,6 +430,7 @@ ext2fs_dent_walk(FS_INFO * fs, INUM_T inode, int flags,
 	dirptr = (char *) ((uintptr_t) dirptr + len);
     }
 
+    fs_inode_free(fs_inode);
     free(dirbuf);
     return;
 }
