@@ -3,7 +3,7 @@
 ** ext2fs_journal
 ** The Sleuth Kit 
 **
-** $Date: 2005/03/25 03:12:34 $
+** $Date: 2005/06/13 19:27:16 $
 **
 ** Journaling code for EXT3FS image
 **
@@ -49,7 +49,7 @@
 
 /* Everything in the journal is in big endian */
 #define big_getu32(x)	\
-	((((uint8_t *)x)[3] <<  0) + \
+	(uint32_t)((((uint8_t *)x)[3] <<  0) + \
 	(((uint8_t *)x)[2] <<  8) + \
 	(((uint8_t *)x)[1] << 16) + \
 	(((uint8_t *)x)[0] << 24) )
@@ -57,8 +57,8 @@
 
 
 static uint8_t
-load_sb_action(FS_INFO * fs, DADDR_T addr, char *buf, int size, int flags,
-	       void *ptr)
+load_sb_action(FS_INFO * fs, DADDR_T addr, char *buf, unsigned int size,
+	       int flags, void *ptr)
 {
     ext2fs_journ_sb *sb;
     EXT2FS_INFO *ext2fs = (EXT2FS_INFO *) fs;
