@@ -2,7 +2,7 @@
 ** fatfs
 ** The Sleuth Kit 
 **
-** $Date: 2005/03/25 03:12:34 $
+** $Date: 2005/07/08 17:20:10 $
 **
 ** Content and meta data layer support for the FAT file system 
 **
@@ -71,7 +71,7 @@ getFAT(FATFS_INFO * fatfs, DADDR_T clust)
 	/* Load the FAT if we don't have it */
 	if ((sect < fatfs->table->addr) ||
 	    (sect >= (fatfs->table->addr + FAT_CACHE_S)) ||
-	    (-1 == fatfs->table->addr)) {
+	    (0 == fatfs->table->used)) {
 	    if (fs_read_block(fs, fatfs->table, FAT_CACHE_B, sect) !=
 		FAT_CACHE_B) {
 		error("getFAT: Error reading FAT12 FAT: %" PRIuDADDR
@@ -99,7 +99,7 @@ getFAT(FATFS_INFO * fatfs, DADDR_T clust)
 	}
 
 	/* get pointer to entry in current buffer */
-	ptr = fatfs->table->data + offs;
+	ptr = (uint8_t *) fatfs->table->data + offs;
 
 	tmp16 = getu16(fs, ptr);
 
@@ -122,7 +122,7 @@ getFAT(FATFS_INFO * fatfs, DADDR_T clust)
 	sect = fatfs->firstfatsect + (clust * 2) / fatfs->ssize;
 	if ((sect < fatfs->table->addr) ||
 	    (sect >= (fatfs->table->addr + FAT_CACHE_S)) ||
-	    (-1 == fatfs->table->addr)) {
+	    (0 == fatfs->table->used)) {
 	    if (fs_read_block(fs, fatfs->table, FAT_CACHE_B, sect) !=
 		FAT_CACHE_B) {
 		error("getFAT: Error reading FAT16 FAT: %" PRIuDADDR
@@ -131,7 +131,7 @@ getFAT(FATFS_INFO * fatfs, DADDR_T clust)
 	}
 
 	/* get pointer to entry in current buffer */
-	ptr = fatfs->table->data +
+	ptr = (uint8_t *) fatfs->table->data +
 	    (sect - fatfs->table->addr) * fatfs->ssize +
 	    (clust * 2) % fatfs->ssize;
 
@@ -149,7 +149,7 @@ getFAT(FATFS_INFO * fatfs, DADDR_T clust)
 	sect = fatfs->firstfatsect + (clust * 4) / fatfs->ssize;
 	if ((sect < fatfs->table->addr) ||
 	    (sect >= (fatfs->table->addr + FAT_CACHE_S)) ||
-	    (-1 == fatfs->table->addr)) {
+	    (0 == fatfs->table->used)) {
 	    if (fs_read_block(fs, fatfs->table, FAT_CACHE_B, sect) !=
 		FAT_CACHE_B) {
 		error("getFAT: Error reading FAT32 FAT: %" PRIuDADDR
@@ -159,7 +159,7 @@ getFAT(FATFS_INFO * fatfs, DADDR_T clust)
 
 
 	/* get pointer to entry in current buffer */
-	ptr = fatfs->table->data +
+	ptr = (uint8_t *) fatfs->table->data +
 	    (sect - fatfs->table->addr) * fatfs->ssize +
 	    (clust * 4) % fatfs->ssize;
 
@@ -173,10 +173,12 @@ getFAT(FATFS_INFO * fatfs, DADDR_T clust)
 	break;
     }
 
+/* This produces way too many entries 
     if (verbose)
 	fprintf(logfp,
 		"fatfs_getFAT: Lookup Cluster %" PRIuDADDR " to %" PRIu32
 		"\n", clust, retval);
+*/
 
     return retval;
 }
@@ -536,15 +538,15 @@ dos2unixmode(uint16_t attr)
 }
 
 
-/*
+/* 
  * Copy the contents of a dentry into a FS_INFO structure
  *
  * The addr is the sector address of where the inode is from.  It is 
  * needed to determine the allocation status.
  */
 void
-fatfs_copy_inode(FATFS_INFO * fatfs, FS_INODE * fs_inode,
-		 fatfs_dentry * in, DADDR_T sect)
+fatfs_dinode_copy(FATFS_INFO * fatfs, FS_INODE * fs_inode,
+		  fatfs_dentry * in, DADDR_T sect, INUM_T inum)
 {
     int dcnt;
     FS_INFO *fs = (FS_INFO *) & fatfs->fs_info;
@@ -555,6 +557,8 @@ fatfs_copy_inode(FATFS_INFO * fatfs, FS_INODE * fs_inode,
     fs_inode->uid = 0;
     fs_inode->gid = 0;
 
+    fs_inode->addr = inum;
+
     /* There is no notion of link in FAT, just deleted or not */
     if ((in->attrib & FATFS_ATTR_LFN) == FATFS_ATTR_LFN) {
 	fs_inode->nlink = 0;
@@ -735,11 +739,14 @@ void
 fatfs_make_root(FATFS_INFO * fatfs, FS_INODE * fs_inode)
 {
     int i;
+    FS_INFO *fs = (FS_INFO *) fatfs;
 
     fs_inode->mode = (FS_INODE_DIR);
 
     fs_inode->nlink = 1;
 
+    fs_inode->addr = fs->root_inum;
+
     fs_inode->flags = (FS_FLAG_META_USED | FS_FLAG_META_ALLOC);
 
     fs_inode->uid = fs_inode->gid = 0;
@@ -849,6 +856,32 @@ fatfs_isdentry(FATFS_INFO * fatfs, fatfs_dentry * de)
  * INODE WALKING
  * 
  *************************************************************************/
+/* Mark the sector used in the bitmap */
+static uint8_t
+inode_walk_file_act(FS_INFO * fs, DADDR_T addr, char *buf,
+		    unsigned int size, int flags, void *ptr)
+{
+    setbit((uint8_t *) ptr, addr);
+    return WALK_CONT;
+}
+
+/* The inode_walk call back for each file.  we want only the directories */
+static uint8_t
+inode_walk_dent_act(FS_INFO * fs, FS_DENT * fs_dent, int flags, void *ptr)
+{
+    if ((fs_dent->fsi == NULL)
+	|| ((fs_dent->fsi->mode & FS_INODE_FMT) != FS_INODE_DIR))
+	return WALK_CONT;
+
+    /* Get the sector addresses */
+    fs->file_walk(fs, fs_dent->fsi, 0, 0,
+		  FS_FLAG_FILE_SLACK | FS_FLAG_FILE_AONLY |
+		  FS_FLAG_FILE_NOABORT | FS_FLAG_FILE_RECOVER,
+		  inode_walk_file_act, ptr);
+
+    return WALK_CONT;
+}
+
 /*
  * walk the inodes
  *
@@ -869,7 +902,7 @@ fatfs_inode_walk(FS_INFO * fs, INUM_T start_inum, INUM_T end_inum,
     DADDR_T sect, ssect, lsect, base_read;
     fatfs_dentry *dep;
     unsigned int myflags, didx, i;
-
+    uint8_t *sect_alloc;
 
 
     /*
@@ -898,7 +931,7 @@ fatfs_inode_walk(FS_INFO * fs, INUM_T start_inum, INUM_T end_inum,
 	int myflags = FS_FLAG_META_ALLOC | FS_FLAG_META_USED;
 
 	fatfs_make_root(fatfs, fs_inode);
-	if (WALK_STOP == action(fs, start_inum, fs_inode, myflags, ptr)) {
+	if (WALK_STOP == action(fs, fs_inode, myflags, ptr)) {
 	    fs_inode_free(fs_inode);
 	    return;
 	}
@@ -913,6 +946,21 @@ fatfs_inode_walk(FS_INFO * fs, INUM_T start_inum, INUM_T end_inum,
     if (start_inum == fs->root_inum)
 	start_inum++;
 
+    /* We are going to be looking at each sector to see if it has 
+     * dentries.  First, run dent_walk to find all sectors that are 
+     * from allocated directories.  We'll be make sure to print those */
+    sect_alloc = (uint8_t *) mymalloc((fs->block_count + 7) / 8);
+    memset((char *) sect_alloc, 0, sizeof(*sect_alloc));
+
+    if (verbose)
+	fprintf(logfp,
+		"fatfs_inode_walk: Walking directories to collect sector info\n");
+
+    fatfs_dent_walk(fs, fs->root_inum,
+		    FS_FLAG_NAME_ALLOC | FS_FLAG_NAME_RECURSE,
+		    inode_walk_dent_act, (void *) sect_alloc);
+
+
     /* As FAT does not give numbers to the directory entries, we will make
      * them up.  Start from one larger then the root inode number (which we
      * made up) and number each entry in each cluster
@@ -925,10 +973,7 @@ fatfs_inode_walk(FS_INFO * fs, INUM_T start_inum, INUM_T end_inum,
      * to analyze it.  If not, then read the next sector 
      */
 
-
-    /* identify the sector numbers that the starting and ending inodes are 
-     * in
-     */
+    /* identify the starting and ending inodes sector addrs */
     ssect = FATFS_INODE_2_SECT(fatfs, start_inum);
     lsect = FATFS_INODE_2_SECT(fatfs, end_inum);
 
@@ -960,8 +1005,12 @@ fatfs_inode_walk(FS_INFO * fs, INUM_T start_inum, INUM_T end_inum,
 		      ": %m", sect);
 	    }
 
-	    if (fatfs_isdentry(fatfs,
-			       (fatfs_dentry *) fatfs->dinodes->data) == 0)
+	    /* if we know it is not part of a directory and it is not valid dentires,
+	     * then skip it */
+	    if ((isset(sect_alloc, sect) == 0) &&
+		(fatfs_isdentry(fatfs,
+				(fatfs_dentry *) fatfs->dinodes->data) ==
+		 0))
 		continue;
 
 	    /* get the base inode address of this sector */
@@ -1030,7 +1079,7 @@ fatfs_inode_walk(FS_INFO * fs, INUM_T start_inum, INUM_T end_inum,
 		if (0 == fatfs_isdentry(fatfs, dep))
 		    continue;
 
-		fatfs_copy_inode(fatfs, fs_inode, dep, sect);
+		fatfs_dinode_copy(fatfs, fs_inode, dep, sect, inum);
 		fs_inode->flags = myflags;
 
 		if (verbose)
@@ -1039,7 +1088,7 @@ fatfs_inode_walk(FS_INFO * fs, INUM_T start_inum, INUM_T end_inum,
 			    " (%u) at sector %" PRIuDADDR "\n", inum, didx,
 			    sect);
 
-		if (WALK_STOP == action(fs, inum, fs_inode, myflags, ptr)) {
+		if (WALK_STOP == action(fs, fs_inode, myflags, ptr)) {
 		    fs_inode_free(fs_inode);
 		    return;
 		}
@@ -1055,13 +1104,11 @@ fatfs_inode_walk(FS_INFO * fs, INUM_T start_inum, INUM_T end_inum,
     base_read =
 	FATFS_CLUST_2_SECT(fatfs, (FATFS_SECT_2_CLUST(fatfs, sect)));
 
-
-
     /* cycle through the sectors and look for dentries
      * Read by cluster since they are bigger and more effecient
      */
     for (sect = base_read; sect <= lsect; sect += fatfs->csize) {
-	int clustalloc, read_size;
+	unsigned int clustalloc, read_size;
 
 	/* if the cluster is not allocated, then do not go into it if we 
 	 * only want allocated/link entries
@@ -1085,16 +1132,16 @@ fatfs_inode_walk(FS_INFO * fs, INUM_T start_inum, INUM_T end_inum,
 		  ": %m", sect);
 	}
 
-
 	for (i = 0; i < read_size; i++) {
 
-
-	    /* if it is not a bunch of dentries, then skip it */
-	    if (0 ==
-		fatfs_isdentry(fatfs,
-			       (fatfs_dentry *) & fatfs->dinodes->data[i *
-								       fatfs->
-								       ssize]))
+	    /* if we know it is not part of a directory and it is not valid dentires,
+	     * then skip it */
+	    if ((isset(sect_alloc, sect) == 0) &&
+		(fatfs_isdentry(fatfs,
+				(fatfs_dentry *) & fatfs->dinodes->data[i *
+									fatfs->
+									ssize])
+		 == 0))
 		continue;
 
 	    /* See if the last inode in this block is smaller than the starting */
@@ -1127,7 +1174,6 @@ fatfs_inode_walk(FS_INFO * fs, INUM_T start_inum, INUM_T end_inum,
 		    return;
 		}
 
-
 		/* if this is a long file name entry, then skip it and 
 		 * wait for the short name */
 		if ((dep->attrib & FATFS_ATTR_LFN) == FATFS_ATTR_LFN)
@@ -1172,7 +1218,7 @@ fatfs_inode_walk(FS_INFO * fs, INUM_T start_inum, INUM_T end_inum,
 		if (0 == fatfs_isdentry(fatfs, dep))
 		    continue;
 
-		fatfs_copy_inode(fatfs, fs_inode, dep, sect);
+		fatfs_dinode_copy(fatfs, fs_inode, dep, sect, inum);
 		fs_inode->flags = myflags;
 
 		if (verbose)
@@ -1181,19 +1227,16 @@ fatfs_inode_walk(FS_INFO * fs, INUM_T start_inum, INUM_T end_inum,
 			    " (%u) at sector %" PRIuDADDR "\n", inum, didx,
 			    sect + i);
 
-		if (WALK_STOP == action(fs, inum, fs_inode, myflags, ptr)) {
+		if (WALK_STOP == action(fs, fs_inode, myflags, ptr)) {
 		    fs_inode_free(fs_inode);
 		    return;
 		}
 	    }
-
 	}			/* dentries */
-
     }				/* clusters */
 
     fs_inode_free(fs_inode);
     return;
-
 }				/* end of inode_walk */
 
 
@@ -1241,7 +1284,7 @@ fatfs_inode_lookup(FS_INFO * fs, INUM_T inum)
 
 	fatfs->dep = (fatfs_dentry *) & fatfs->dinodes->data[off];
 	if (fatfs_isdentry(fatfs, fatfs->dep)) {
-	    fatfs_copy_inode(fatfs, fs_inode, fatfs->dep, sect);
+	    fatfs_dinode_copy(fatfs, fs_inode, fatfs->dep, sect, inum);
 	}
 	else {
 	    error("%" PRIuINUM " is not an inode", inum);
@@ -1275,7 +1318,7 @@ fatfs_file_walk(FS_INFO * fs, FS_INODE * fs_inode, uint32_t type,
 		uint16_t id, int flags, FS_FILE_WALK_FN action, void *ptr)
 {
     FATFS_INFO *fatfs = (FATFS_INFO *) fs;
-    int i;
+    unsigned int i;
     OFF_T size;
     DADDR_T clust, sbase;
     uint32_t len;
@@ -1339,6 +1382,10 @@ fatfs_file_walk(FS_INFO * fs, FS_INODE * fs_inode, uint32_t type,
 	OFF_T recoversize = size;
 
 
+	if (verbose)
+	    fprintf(logfp,
+		    "fatfs_file_walk: Walking deleted file %" PRIuINUM
+		    " in recovery mode\n", fs_inode->addr);
 
 	/* We know the size and the starting cluster
 	 *
@@ -1432,7 +1479,7 @@ fatfs_file_walk(FS_INFO * fs, FS_INODE * fs_inode, uint32_t type,
 	    if ((flags & FS_FLAG_FILE_AONLY) == 0) {
 		if (fs_read_block
 		    (fs, data_buf, fatfs->ssize * fatfs->csize, sbase)
-		    != fatfs->ssize * fatfs->csize) {
+		    != (OFF_T) (fatfs->ssize * fatfs->csize)) {
 		    error
 			("fatfs_file_walk: Error reading block during recovery: %"
 			 PRIuDADDR ": %m", sbase);
@@ -1462,6 +1509,11 @@ fatfs_file_walk(FS_INFO * fs, FS_INODE * fs_inode, uint32_t type,
 
     /* Normal cluster chain walking */
     else {
+
+	if (verbose)
+	    fprintf(logfp,
+		    "fatfs_file_walk: Walking file %" PRIuINUM
+		    " in normal mode\n", fs_inode->addr);
 	/* Cycle through the cluster chain */
 	while ((clust & fatfs->mask) > 0 && size > 0 &&
 	       (0 == FATFS_ISEOF(clust, fatfs->mask))) {
@@ -1498,7 +1550,7 @@ fatfs_file_walk(FS_INFO * fs, FS_INODE * fs_inode, uint32_t type,
 	    if ((flags & FS_FLAG_FILE_AONLY) == 0) {
 		if (fs_read_block
 		    (fs, data_buf, fatfs->ssize * fatfs->csize,
-		     sbase) != fatfs->ssize * fatfs->csize) {
+		     sbase) != (OFF_T) (fatfs->ssize * fatfs->csize)) {
 		    error("fatfs_file_walk: Error reading sector: %"
 			  PRIuDADDR ": %m", sbase);
 		}
@@ -1563,7 +1615,8 @@ fatfs_fscheck(FS_INFO * fs, FILE * hFile)
 static void
 fatfs_fsstat(FS_INFO * fs, FILE * hFile)
 {
-    int i, a, cnt;
+    unsigned int i;
+    int a, cnt;
     DADDR_T next, snext, sstart, send;
     FATFS_INFO *fatfs = (FATFS_INFO *) fs;
     fatfs_sb *sb = fatfs->sb;
@@ -1892,7 +1945,7 @@ typedef struct {
 
 static uint8_t
 print_addr_act(FS_INFO * fs, DADDR_T addr, char *buf,
-	       int size, int flags, void *ptr)
+	       unsigned int size, int flags, void *ptr)
 {
     FATFS_PRINT_ADDR *print = (FATFS_PRINT_ADDR *) ptr;
 
@@ -2023,6 +2076,8 @@ fatfs_istat(FS_INFO * fs, FILE * hFile, INUM_T inum, int numblock,
 	    fprintf(hFile, "\n");
 
     }
+
+    fs_inode_free(fs_inode);
     return;
 }
 
@@ -2070,7 +2125,7 @@ fatfs_open(IMG_INFO * img_info, unsigned char ftype, uint8_t test)
 {
     char *myname = "fatfs_open";
     FATFS_INFO *fatfs = (FATFS_INFO *) mymalloc(sizeof(*fatfs));
-    int len;
+    unsigned int len;
     FS_INFO *fs = &(fatfs->fs_info);
     fatfs_sb *fatsb;
     uint32_t clustcnt, sectors;
@@ -2315,12 +2370,6 @@ fatfs_open(IMG_INFO * img_info, unsigned char ftype, uint8_t test)
     fs->last_inum = fs->first_inum + fs->inum_count;
 
 
-    /* long file name support */
-    fatfs->lfn = NULL;
-    fatfs->lfn_len = 0;
-    fatfs->lfn_chk = 0;
-
-
     /*
      * Other initialization: caches, callbacks.
      */
