@@ -2,7 +2,7 @@
 ** ffind  (file find)
 ** The Sleuth Kit 
 **
-** $Date: 2005/03/25 03:12:34 $
+** $Date: 2005/05/18 00:41:57 $
 **
 ** Find the file that uses the specified inode (including deleted files)
 ** 
@@ -51,10 +51,6 @@
 */
 #include "fs_lib.h"
 
-/* NTFS has an optimized version of this function */
-extern void ntfs_find_file(FS_INFO *, INUM_T, uint32_t, uint32_t, int,
-			   FS_DENT_WALK_FN, char *);
-
 void
 usage()
 {
@@ -90,7 +86,8 @@ main(int argc, char **argv)
     char *cp;
     FS_INFO *fs;
     extern int optind;
-    uint32_t type, id;
+    uint32_t type;
+    uint16_t id;
     char *imgtype = NULL, *imgoff = NULL, *dash;
     IMG_INFO *img;
     uint8_t localflags = 0;
@@ -162,13 +159,13 @@ main(int argc, char **argv)
 	    *dash2 = '\0';
 	    dash2++;
 
-	    id = strtoull(dash2, &cp, 0);
+	    id = (uint16_t) strtoul(dash2, &cp, 0);
 	    if (*cp || cp == dash2) {
 		fprintf(stderr, "Invalid id in inode: %s\n", dash2);
 		usage();
 	    }
 	}
-	type = strtoull(dash, &cp, 0);
+	type = strtoul(dash, &cp, 0);
 	if (*cp || cp == dash) {
 	    fprintf(stderr, "Invalid type in inode: %s\n", dash);
 	    usage();
