@@ -2,7 +2,7 @@
 ** ffind  (file find)
 ** The Sleuth Kit 
 **
-** $Date: 2005/03/25 03:12:34 $
+** $Date: 2005/06/13 19:27:16 $
 **
 ** Find the file that uses the specified inode (including deleted files)
 ** 
@@ -52,9 +52,9 @@
 #include "fs_lib.h"
 
 /* NTFS has an optimized version of this function */
-extern void ntfs_find_file(FS_INFO *, INUM_T, uint32_t, uint32_t, int,
-			   FS_DENT_WALK_FN, char *);
-
+extern void
+ ntfs_find_file(FS_INFO *, INUM_T, uint32_t,
+		uint16_t, int, FS_DENT_WALK_FN, void *ptr);
 
 
 static INUM_T inode = 0;
@@ -82,7 +82,7 @@ find_file_act(FS_INFO * fs, FS_DENT * fs_dent, int flags, void *ptr)
 
 uint8_t
 fs_ffind(FS_INFO * fs, uint8_t lclflags, INUM_T inode_a, uint32_t type,
-	 uint32_t id, int flags)
+	 uint16_t id, int flags)
 {
 
     found = 0;
@@ -121,6 +121,7 @@ fs_ffind(FS_INFO * fs, uint8_t lclflags, INUM_T inode_a, uint32_t type,
 		    printf("* ");
 		printf("%s/%s\n", ORPHAN_STR, fs_inode->name->name);
 	    }
+	    fs_inode_free(fs_inode);
 	}
 	else {
 	    printf("inode not currently used\n");
