@@ -1,7 +1,7 @@
 /*
 ** The Sleuth Kit 
 **
-** $Date: 2005/01/17 22:40:16 $
+** $Date: 2005/07/08 17:20:10 $
 **
 ** Brian Carrier [carrier@sleuthkit.org]
 ** Copyright (c) 2003-2005 Brian Carrier.  All rights reserved 
@@ -28,10 +28,9 @@
 
 
 
-/* ffs_cgroup_lookup - look up cached cylinder group info */
-
-static ffs_cgd *
-ffs_cgroup_lookup(FFS_INFO * ffs, FFS_GRPNUM_T cgnum)
+/* ffs_group_load - load cylinder group descriptor info into cache */
+static void
+ffs_group_load(FFS_INFO * ffs, FFS_GRPNUM_T grp_num)
 {
     DADDR_T addr;
     FS_INFO *fs = (FS_INFO *) & ffs->fs_info;
@@ -39,47 +38,36 @@ ffs_cgroup_lookup(FFS_INFO * ffs, FFS_GRPNUM_T cgnum)
     /*
      * Sanity check
      */
-    if (cgnum < 0 || cgnum >= gets32(fs, ffs->fs.sb1->cg_num))
-	error("invalid cylinder group number: %" PRI_FFSGRP "", cgnum);
+    if (grp_num < 0 || grp_num >= ffs->groups_count)
+	error("invalid cylinder group number: %" PRI_FFSGRP "", grp_num);
 
     /*
      * Allocate/read cylinder group info on the fly. Trust that a cylinder
      * group always fits within a logical disk block (as promised in the
      * 4.4BSD <ufs/ffs/fs.h> include file).
      */
-    if (ffs->cg_buf == NULL)
-	ffs->cg_buf = data_buf_alloc(ffs->ffsbsize_b);
+    if (ffs->grp_buf == NULL)
+	ffs->grp_buf = data_buf_alloc(ffs->ffsbsize_b);
 
-    addr = cgtod_lcl(fs, ffs->fs.sb1, cgnum);
-    if (ffs->cg_buf->addr != addr) {
-	if (fs_read_block(fs, ffs->cg_buf, ffs->cg_buf->size, addr) !=
-	    ffs->cg_buf->size) {
-	    error("ffs_cgroup_lookup: Error reading group %d at %"
-		  PRIuDADDR ":  %m", cgnum, addr);
+    addr = cgtod_lcl(fs, ffs->fs.sb1, grp_num);
+    if (ffs->grp_buf->addr != addr) {
+	if (fs_read_block(fs, ffs->grp_buf, ffs->grp_buf->size, addr) !=
+	    ffs->grp_buf->size) {
+	    error("ffs_group_load: Error reading group %d at %"
+		  PRIuDADDR ":  %m", grp_num, addr);
 	}
-
     }
 
-    ffs->cg_num = cgnum;
-
-    return (ffs_cgd *) ffs->cg_buf->data;
+    ffs->grp_num = grp_num;
 }
 
-/* ffs_cgroup_free - destroy cylinder group info cache */
-static void
-ffs_cgroup_free(FFS_INFO * ffs)
-{
-    if (ffs->cg_buf)
-	data_buf_free(ffs->cg_buf);
-}
 
 /* 
- * ffs_dinode_lookup - look up cached disk inode 
- * Place result in ffs->dinode
+ * ffs_dinode_load - read disk inode and load into local cache (ffs->dino_buf)
  */
 
 static void
-ffs_dinode_lookup(FFS_INFO * ffs, INUM_T inum)
+ffs_dinode_load(FFS_INFO * ffs, INUM_T inum)
 {
     DADDR_T addr;
     OFF_T offs;
@@ -92,15 +80,10 @@ ffs_dinode_lookup(FFS_INFO * ffs, INUM_T inum)
 	error("invalid inode number: %" PRIuINUM "", inum);
 
     /*
-     * Allocate/read the inode buffer on the fly.
-     */
-    if (ffs->dino_buf == 0)
-	ffs->dino_buf = data_buf_alloc(ffs->ffsbsize_b);
-
-
-    /*
-     * Copy the inode into the buffer
+     * Allocate/read the inode table buffer on the fly.
      */
+    if (ffs->itbl_buf == NULL)
+	ffs->itbl_buf = data_buf_alloc(ffs->ffsbsize_b);
 
 
     /* UFS2 is different because it does not initialize all inodes
@@ -110,95 +93,93 @@ ffs_dinode_lookup(FFS_INFO * ffs, INUM_T inum)
      */
     if (fs->ftype == FFS_2) {
 	ffs_cgd2 *cg2;
-	FFS_GRPNUM_T cg_num = itog_lcl(fs, ffs->fs.sb1, inum);
+	FFS_GRPNUM_T grp_num;
+
+	if (ffs->dino_buf == NULL)
+	    ffs->dino_buf = (char *) mymalloc(sizeof(ffs_inode2));
+	else if (ffs->dino_inum == inum)
+	    return;
 
 	/* Lookup the cylinder group descriptor if it isn't
 	 * cached
 	 */
-	if ((ffs->cg_buf == NULL) || (cg_num != ffs->cg_num)) {
-	    cg2 = (ffs_cgd2 *) ffs_cgroup_lookup(ffs, cg_num);
-	}
-	else {
-	    cg2 = (ffs_cgd2 *) ffs->cg_buf->data;
-	}
+	grp_num = itog_lcl(fs, ffs->fs.sb1, inum);
+	if ((ffs->grp_buf == NULL) || (grp_num != ffs->grp_num))
+	    ffs_group_load(ffs, grp_num);
 
+	cg2 = (ffs_cgd2 *) ffs->grp_buf->data;
 
 	/* If the inode is not init, then do not worry about it */
-	if ((inum - cg_num * getu32(fs, ffs->fs.sb2->cg_inode_num)) >=
+	if ((inum - grp_num * getu32(fs, ffs->fs.sb2->cg_inode_num)) >=
 	    getu32(fs, cg2->cg_initediblk)) {
-	    memset((char *) ffs->dinode, 0, sizeof(ffs_inode2));
+	    memset((char *) ffs->dino_buf, 0, sizeof(ffs_inode2));
 	}
 
 	else {
 	    /* Get the base and offset addr for the inode in the tbl */
 	    addr = itod_lcl(fs, ffs->fs.sb1, inum);
 
-	    if (ffs->dino_buf->addr != addr) {
+	    if (ffs->itbl_buf->addr != addr) {
 		if (fs_read_block
-		    (fs, ffs->dino_buf, ffs->dino_buf->size,
-		     addr) != ffs->dino_buf->size) {
+		    (fs, ffs->itbl_buf, ffs->itbl_buf->size,
+		     addr) != ffs->itbl_buf->size) {
 		    error
-			("ffs_dinode_lookup: Error reading FFS2 inode table at %"
+			("ffs_dinode_load: Error reading FFS2 inode table at %"
 			 PRIuDADDR ":  %m", addr);
 		}
 	    }
 
 	    offs = itoo_lcl(fs, ffs->fs.sb2, inum) * sizeof(ffs_inode2);
 
-	    memcpy((char *) ffs->dinode, ffs->dino_buf->data + offs,
+	    memcpy((char *) ffs->dino_buf, ffs->itbl_buf->data + offs,
 		   sizeof(ffs_inode2));
 	}
     }
     else {
+	if (ffs->dino_buf == NULL)
+	    ffs->dino_buf = (char *) mymalloc(sizeof(ffs_inode1));
+	else if (ffs->dino_inum == inum)
+	    return;
+
 	addr = itod_lcl(fs, ffs->fs.sb1, inum);
-	if (ffs->dino_buf->addr != addr) {
-	    if (fs_read_block(fs, ffs->dino_buf, ffs->dino_buf->size, addr)
-		!= ffs->dino_buf->size) {
+	if (ffs->itbl_buf->addr != addr) {
+	    if (fs_read_block(fs, ffs->itbl_buf, ffs->itbl_buf->size, addr)
+		!= ffs->itbl_buf->size) {
 		error
-		    ("ffs_dinode_lookup: Error reading FFS1 inode table at %"
+		    ("ffs_dinode_load: Error reading FFS1 inode table at %"
 		     PRIuDADDR ":  %m", addr);
 	    }
 	}
 
 	offs = itoo_lcl(fs, ffs->fs.sb1, inum) * sizeof(ffs_inode1);
 
-	memcpy((char *) ffs->dinode, ffs->dino_buf->data + offs,
+	memcpy((char *) ffs->dino_buf, ffs->itbl_buf->data + offs,
 	       sizeof(ffs_inode1));
     }
 
-    ffs->inum = inum;
+    ffs->dino_inum = inum;
 
     return;
 }
 
-/* ffs_dinode_free - destroy disk inode cache */
-static void
-ffs_dinode_free(FFS_INFO * ffs)
-{
-    if (ffs->dino_buf)
-	data_buf_free(ffs->dino_buf);
-
-    ffs->inum = 0;
-}
 
 
-/* ffs_copy_inode - copy disk inode to generic inode 
- * 
+/* ffs_dinode_copy - copy cached disk inode to generic inode  
  */
-
-
 static void
-ffs_copy_inode(FFS_INFO * ffs, FS_INODE * fs_inode)
+ffs_dinode_copy(FFS_INFO * ffs, FS_INODE * fs_inode)
 {
-    int i, j, count;
+    int i, j;
+    unsigned int count;
     FS_INFO *fs = &(ffs->fs_info);
-    FFS_GRPNUM_T cg_num;
+    FFS_GRPNUM_T grp_num;
     ffs_cgd *cg;
     unsigned char *inosused = NULL;
     INUM_T ibase;
 
     fs_inode->flags = 0;
     fs_inode->seq = 0;
+    fs_inode->addr = ffs->dino_inum;
 
     /* If the symlink field is set from a previous run, then free it */
     if (fs_inode->link) {
@@ -208,7 +189,7 @@ ffs_copy_inode(FFS_INFO * ffs, FS_INODE * fs_inode)
 
     /* OpenBSD and FreeBSD style */
     if (fs->ftype == FFS_1) {
-	ffs_inode1 *in = (ffs_inode1 *) ffs->dinode;
+	ffs_inode1 *in = (ffs_inode1 *) ffs->dino_buf;
 
 	fs_inode->mode = getu16(fs, in->di_mode);
 	fs_inode->nlink = gets16(fs, in->di_nlink);
@@ -291,7 +272,7 @@ ffs_copy_inode(FFS_INFO * ffs, FS_INODE * fs_inode)
 				      fs_inode->direct_addr[i]) !=
 			fs->block_size) {
 			error
-			    ("ffs_copy_inode: Error reading FFS1A symlink destination at %"
+			    ("ffs_dinode_copy: Error reading FFS1A symlink destination at %"
 			     PRIuDADDR ":  %m", fs_inode->direct_addr[i]);
 		    }
 
@@ -308,7 +289,7 @@ ffs_copy_inode(FFS_INFO * ffs, FS_INODE * fs_inode)
     }
     /* FFS_1B - Solaris */
     else if (fs->ftype == FFS_1B) {
-	ffs_inode1b *in = (ffs_inode1b *) ffs->dinode;
+	ffs_inode1b *in = (ffs_inode1b *) ffs->dino_buf;
 
 	fs_inode->mode = getu16(fs, in->di_mode);
 	fs_inode->nlink = gets16(fs, in->di_nlink);
@@ -354,7 +335,7 @@ ffs_copy_inode(FFS_INFO * ffs, FS_INODE * fs_inode)
 				  fs_inode->direct_addr[i]) !=
 		    fs->block_size) {
 		    error
-			("ffs_copy_inode: Error reading FFS1B symlink destination at %"
+			("ffs_dinode_copy: Error reading FFS1B symlink destination at %"
 			 PRIuDADDR ":  %m", fs_inode->direct_addr[i]);
 		}
 
@@ -370,7 +351,7 @@ ffs_copy_inode(FFS_INFO * ffs, FS_INODE * fs_inode)
 	}
     }
     else if (fs->ftype == FFS_2) {
-	ffs_inode2 *in = (ffs_inode2 *) ffs->dinode;
+	ffs_inode2 *in = (ffs_inode2 *) ffs->dino_buf;
 
 	fs_inode->mode = getu16(fs, in->di_mode);
 	fs_inode->nlink = gets16(fs, in->di_nlink);
@@ -453,7 +434,7 @@ ffs_copy_inode(FFS_INFO * ffs, FS_INODE * fs_inode)
 				      fs_inode->direct_addr[i]) !=
 			fs->block_size) {
 			error
-			    ("ffs_copy_inode: Error reading FFS2 symlink destination at %"
+			    ("ffs_dinode_copy: Error reading FFS2 symlink destination at %"
 			     PRIuDADDR ":  %m", fs_inode->direct_addr[i]);
 		    }
 
@@ -473,13 +454,17 @@ ffs_copy_inode(FFS_INFO * ffs, FS_INODE * fs_inode)
     }
 
     /* set the flags */
-    cg_num = itog_lcl(fs, ffs->fs.sb1, ffs->inum);
-    cg = ffs_cgroup_lookup(ffs, cg_num);
+    grp_num = itog_lcl(fs, ffs->fs.sb1, ffs->dino_inum);
+    if ((ffs->grp_buf == NULL) || (grp_num != ffs->grp_num))
+	ffs_group_load(ffs, grp_num);
+
+    cg = (ffs_cgd *) ffs->grp_buf->data;
+
     inosused = (unsigned char *) cg_inosused_lcl(fs, cg);
-    ibase = cg_num * gets32(fs, ffs->fs.sb1->cg_inode_num);
+    ibase = grp_num * gets32(fs, ffs->fs.sb1->cg_inode_num);
 
     /* get the alloc flag */
-    fs_inode->flags = (isset(inosused, ffs->inum - ibase) ?
+    fs_inode->flags = (isset(inosused, ffs->dino_inum - ibase) ?
 		       FS_FLAG_META_ALLOC : FS_FLAG_META_UNALLOC);
 
     /* link flag */
@@ -490,12 +475,11 @@ ffs_copy_inode(FFS_INFO * ffs, FS_INODE * fs_inode)
     fs_inode->flags |= (fs_inode->ctime ?
 			FS_FLAG_META_USED : FS_FLAG_META_UNUSED);
 
-}				/* end of ffs_copy_inode */
+}
 
 
 
 /* ffs_inode_lookup - lookup inode, external interface */
-
 static FS_INODE *
 ffs_inode_lookup(FS_INFO * fs, INUM_T inum)
 {
@@ -503,15 +487,14 @@ ffs_inode_lookup(FS_INFO * fs, INUM_T inum)
     FS_INODE *fs_inode;
 
     /* Lookup the inode and store it in ffs */
-    ffs_dinode_lookup(ffs, inum);
+    ffs_dinode_load(ffs, inum);
 
     /* copy it to the FS_INODE structure */
     fs_inode = fs_inode_alloc(FFS_NDADDR, FFS_NIADDR);
-    ffs_copy_inode(ffs, fs_inode);
+    ffs_dinode_copy(ffs, fs_inode);
 
     return (fs_inode);
-
-}				/* end of ffs_inode_lookup */
+}
 
 
 
@@ -533,8 +516,8 @@ ffs_inode_walk(FS_INFO * fs, INUM_T start_inum, INUM_T end_inum, int flags,
 {
     char *myname = "ffs_inode_walk";
     FFS_INFO *ffs = (FFS_INFO *) fs;
-    FFS_GRPNUM_T cg_num;
-    ffs_cgd *cg = 0;
+    FFS_GRPNUM_T grp_num;
+    ffs_cgd *cg = NULL;
     INUM_T inum;
     unsigned char *inosused = NULL;
     FS_INODE *fs_inode = fs_inode_alloc(FFS_NDADDR, FFS_NIADDR);
@@ -561,12 +544,18 @@ ffs_inode_walk(FS_INFO * fs, INUM_T start_inum, INUM_T end_inum, int flags,
 	/*
 	 * Be sure to use the proper cylinder group data.
 	 */
-	cg_num = itog_lcl(fs, ffs->fs.sb1, inum);
+	grp_num = itog_lcl(fs, ffs->fs.sb1, inum);
 
-	if (cg_num != ffs->cg_num) {
-	    cg = ffs_cgroup_lookup(ffs, cg_num);
+	if ((ffs->grp_buf == NULL) || (grp_num != ffs->grp_num)) {
+	    ffs_group_load(ffs, grp_num);
+	    cg = NULL;
+	}
+
+	/* Load up the cached one if the needed one was already loaded or if a new was just loaded */
+	if (cg == NULL) {
+	    cg = (ffs_cgd *) ffs->grp_buf->data;
 	    inosused = (unsigned char *) cg_inosused_lcl(fs, cg);
-	    ibase = cg_num * gets32(fs, ffs->fs.sb1->cg_inode_num);
+	    ibase = grp_num * gets32(fs, ffs->fs.sb1->cg_inode_num);
 	}
 
 	/*
@@ -577,12 +566,12 @@ ffs_inode_walk(FS_INFO * fs, INUM_T start_inum, INUM_T end_inum, int flags,
 	if ((flags & myflags) != myflags)
 	    continue;
 
-	ffs_dinode_lookup(ffs, inum);
+	ffs_dinode_load(ffs, inum);
 
 
 	if ((fs->ftype == FFS_1) || (fs->ftype == FFS_1B)) {
 	    /* both inode forms are the same for the required fields */
-	    ffs_inode1 *in1 = (ffs_inode1 *) ffs->dinode;
+	    ffs_inode1 *in1 = (ffs_inode1 *) ffs->dino_buf;
 
 	    /*
 	     * Apply the linked/unlinked restriction.
@@ -601,7 +590,7 @@ ffs_inode_walk(FS_INFO * fs, INUM_T start_inum, INUM_T end_inum, int flags,
 		continue;
 	}
 	else {
-	    ffs_inode2 *in2 = (ffs_inode2 *) ffs->dinode;
+	    ffs_inode2 *in2 = (ffs_inode2 *) ffs->dino_buf;
 
 	    /*
 	     * Apply the linked/unlinked restriction.
@@ -624,9 +613,9 @@ ffs_inode_walk(FS_INFO * fs, INUM_T start_inum, INUM_T end_inum, int flags,
 	 * Fill in a file system-independent inode structure and pass control
 	 * to the application.
 	 */
-	ffs_copy_inode(ffs, fs_inode);
+	ffs_dinode_copy(ffs, fs_inode);
 	fs_inode->flags = myflags;
-	if (WALK_STOP == action(fs, inum, fs_inode, myflags, ptr)) {
+	if (WALK_STOP == action(fs, fs_inode, myflags, ptr)) {
 	    fs_inode_free(fs_inode);
 	    return;
 	}
@@ -658,7 +647,7 @@ ffs_block_walk(FS_INFO * fs, DADDR_T start_blk, DADDR_T end_blk, int flags,
     char *myname = "ffs_block_walk";
     FFS_INFO *ffs = (FFS_INFO *) fs;
     DATA_BUF *data_buf = data_buf_alloc(fs->block_size * ffs->ffsbsize_f);
-    FFS_GRPNUM_T cg_num;
+    FFS_GRPNUM_T grp_num;
     ffs_cgd *cg = 0;
     DADDR_T dbase = 0;
     DADDR_T dmin = 0;		/* first data block in group */
@@ -709,17 +698,18 @@ ffs_block_walk(FS_INFO * fs, DADDR_T start_blk, DADDR_T end_blk, int flags,
 	/*
 	 * Be sure to use the right cylinder group information.
 	 */
-	cg_num = dtog_lcl(fs, ffs->fs.sb1, addr);
-	if (cg == 0 || gets32(fs, cg->cg_cgx) != cg_num) {
-	    cg = ffs_cgroup_lookup(ffs, cg_num);
+	grp_num = dtog_lcl(fs, ffs->fs.sb1, addr);
+	if (cg == 0 || (unsigned int) gets32(fs, cg->cg_cgx) != grp_num) {
+	    ffs_group_load(ffs, grp_num);
+	    cg = (ffs_cgd *) ffs->grp_buf->data;
 	    freeblocks = (unsigned char *) cg_blksfree_lcl(fs, cg);
-	    dbase = cgbase_lcl(fs, ffs->fs.sb1, cg_num);
-	    dmin = cgdmin_lcl(fs, ffs->fs.sb1, cg_num);
-	    sblock = cgsblock_lcl(fs, ffs->fs.sb1, cg_num);
+	    dbase = cgbase_lcl(fs, ffs->fs.sb1, grp_num);
+	    dmin = cgdmin_lcl(fs, ffs->fs.sb1, grp_num);
+	    sblock = cgsblock_lcl(fs, ffs->fs.sb1, grp_num);
 	}
 	if (addr < dbase)
 	    remark("impossible: cyl group %lu: block %lu < cgbase %lu",
-		   (unsigned long) cg_num, (unsigned long) addr,
+		   (unsigned long) grp_num, (unsigned long) addr,
 		   (unsigned long) dbase);
 
 	/*
@@ -890,7 +880,7 @@ ffs_file_walk_indir(FS_INFO * fs, DATA_BUF * buf[], OFF_T length,
 {
     char *myname = "ffs_file_walk_indir";
     OFF_T todo_count = length;
-    int n;
+    unsigned int n;
 
     if (verbose)
 	fprintf(logfp, "%s: level %d block %" PRIuDADDR "\n", myname,
@@ -1060,6 +1050,10 @@ ffs_file_walk(FS_INFO * fs, FS_INODE * inode, uint32_t type, uint16_t id,
     int n;
     int level;
 
+    if (verbose)
+	fprintf(logfp, "ffs_file_walk: Processing file %" PRIuINUM "\n",
+		inode->addr);
+
     /*
      * Initialize a buffer for each level of indirection that is supported by
      * this inode. The level 0 buffer is sized to the logical block size used
@@ -1129,7 +1123,7 @@ ffs_fscheck(FS_INFO * fs, FILE * hFile)
 static void
 ffs_fsstat(FS_INFO * fs, FILE * hFile)
 {
-    int i;
+    unsigned int i;
     time_t tmptime;
     ffs_csum1 *csum1 = NULL;
     ffs_cgd *cgd = NULL;
@@ -1241,8 +1235,8 @@ ffs_fsstat(FS_INFO * fs, FILE * hFile)
     fprintf(hFile, "\nCYLINDER GROUP INFORMATION\n");
     fprintf(hFile, "--------------------------------------------\n");
 
-    fprintf(hFile, "Number of Cylinder Groups: %" PRId32 "\n",
-	    gets32(fs, sb1->cg_num));
+    fprintf(hFile, "Number of Cylinder Groups: %" PRIu32 "\n",
+	    ffs->groups_count);
     fprintf(hFile, "Inodes per group: %" PRId32 "\n",
 	    gets32(fs, sb1->cg_inode_num));
     fprintf(hFile, "Fragments per group: %" PRId32 "\n",
@@ -1278,9 +1272,10 @@ ffs_fsstat(FS_INFO * fs, FILE * hFile)
 	}
     }
 
-    for (i = 0; i < gets32(fs, sb1->cg_num); i++) {
+    for (i = 0; i < ffs->groups_count; i++) {
 
-	cgd = ffs_cgroup_lookup(ffs, i);
+	ffs_group_load(ffs, i);
+	cgd = (ffs_cgd *) ffs->grp_buf->data;
 
 	fprintf(hFile, "\nGroup %d:\n", i);
 	if (cgd) {
@@ -1297,10 +1292,12 @@ ffs_fsstat(FS_INFO * fs, FILE * hFile)
 
 	fprintf(hFile, "  Inode Range: %" PRIu32 " - %" PRIu32 "\n",
 		(gets32(fs, sb1->cg_inode_num) * i),
-		(((gets32(fs, sb1->cg_inode_num) * (i + 1)) - 1) <
-		 fs->last_inum) ? ((gets32(fs, sb1->cg_inode_num) *
-				    (i + 1)) -
-				   1) : (uint32_t) fs->last_inum);
+		((uint32_t) ((gets32(fs, sb1->cg_inode_num) * (i + 1)) - 1)
+		 <
+		 fs->
+		 last_inum) ? (uint32_t) ((gets32(fs, sb1->cg_inode_num) *
+					   (i + 1)) -
+					  1) : (uint32_t) fs->last_inum);
 
 	fprintf(hFile,
 		"  Fragment Range: %" PRIuDADDR " - %" PRIuDADDR "\n",
@@ -1435,7 +1432,7 @@ typedef struct {
 
 static uint8_t
 print_addr_act(FS_INFO * fs, DADDR_T addr, char *buf,
-	       int size, int flags, void *ptr)
+	       unsigned int size, int flags, void *ptr)
 {
     FFS_PRINT_ADDR *print = (FFS_PRINT_ADDR *) ptr;
 
@@ -1481,7 +1478,7 @@ ffs_istat(FS_INFO * fs, FILE * hFile, INUM_T inum, int numblock,
     fprintf(hFile, "%sAllocated\n",
 	    (fs_inode->flags & FS_FLAG_META_ALLOC) ? "" : "Not ");
 
-    fprintf(hFile, "Group: %" PRI_FFSGRP "\n", ffs->cg_num);
+    fprintf(hFile, "Group: %" PRI_FFSGRP "\n", ffs->grp_num);
 
     if (fs_inode->link)
 	fprintf(hFile, "symbolic link to: %s\n", fs_inode->link);
@@ -1522,7 +1519,7 @@ ffs_istat(FS_INFO * fs, FILE * hFile, INUM_T inum, int numblock,
     fprintf(hFile, "Inode Modified:\t%s", ctime(&fs_inode->ctime));
 
     if (fs->ftype == FFS_2) {
-	ffs_inode2 *in = (ffs_inode2 *) ffs->dinode;
+	ffs_inode2 *in = (ffs_inode2 *) ffs->dino_buf;
 	/* Are there extended attributes */
 	if (getu32(fs, in->di_extsize) > 0) {
 	    ffs_extattr *ea;
@@ -1542,7 +1539,7 @@ ffs_istat(FS_INFO * fs, FILE * hFile, INUM_T inum, int numblock,
 	    // @@@ Incorporate values into this as well
 	    if ((getu64(fs, in->di_extb[0]) >= fs->first_block) &&
 		(getu64(fs, in->di_extb[0]) <= fs->last_block)) {
-		int end;
+		uintptr_t end;
 
 		if (fs_read_block(fs, data_buf, ffs->ffsbsize_b,
 				  getu64(fs,
@@ -1575,7 +1572,7 @@ ffs_istat(FS_INFO * fs, FILE * hFile, INUM_T inum, int numblock,
 	    }
 	    if ((getu64(fs, in->di_extb[1]) >= fs->first_block) &&
 		(getu64(fs, in->di_extb[1]) <= fs->last_block)) {
-		int end;
+		uintptr_t end;
 
 		if (fs_read_block(fs, data_buf, ffs->ffsbsize_b,
 				  getu64(fs,
@@ -1633,7 +1630,7 @@ ffs_istat(FS_INFO * fs, FILE * hFile, INUM_T inum, int numblock,
 	printidx = 0;
 
 	for (i = 0; i < print.indir_idx; i++) {
-	    int a;
+	    unsigned int a;
 	    /* Cycle through the fragments in the block */
 	    for (a = 0; a < ffs->ffsbsize_f; a++) {
 		fprintf(hFile, "%" PRIuDADDR " ", print.indirl[i] + a);
@@ -1647,6 +1644,7 @@ ffs_istat(FS_INFO * fs, FILE * hFile, INUM_T inum, int numblock,
 	    fprintf(hFile, "\n");
     }
 
+    fs_inode_free(fs_inode);
     return;
 }
 
@@ -1682,10 +1680,16 @@ ffs_close(FS_INFO * fs)
 {
     FFS_INFO *ffs = (FFS_INFO *) fs;
 
-    ffs_cgroup_free(ffs);
-    ffs_dinode_free(ffs);
+    if (ffs->grp_buf)
+	data_buf_free(ffs->grp_buf);
+
+    if (ffs->itbl_buf)
+	data_buf_free(ffs->itbl_buf);
+
+    if (ffs->dino_buf)
+	free(ffs->dino_buf);
+
     free((char *) ffs->fs.sb1);
-    free(ffs->dinode);
     free(ffs);
 }
 
@@ -1696,7 +1700,7 @@ ffs_open(IMG_INFO * img_info, unsigned char ftype, uint8_t test)
 {
     char *myname = "ffs_open";
     FFS_INFO *ffs = (FFS_INFO *) mymalloc(sizeof(*ffs));
-    int len;
+    unsigned int len;
     FS_INFO *fs = &(ffs->fs_info);
 
     if ((ftype & FSMASK) != FFS_TYPE)
@@ -1772,14 +1776,18 @@ ffs_open(IMG_INFO * img_info, unsigned char ftype, uint8_t test)
 	fs->block_size = gets32(fs, ffs->fs.sb2->fsize_b);
 	ffs->ffsbsize_b = gets32(fs, ffs->fs.sb2->bsize_b);
 	ffs->ffsbsize_f = gets32(fs, ffs->fs.sb2->bsize_frag);
+	ffs->groups_count = gets32(fs, ffs->fs.sb2->cg_num);
     }
     else {
 	fs->block_count = gets32(fs, ffs->fs.sb1->frag_num);
 	fs->block_size = gets32(fs, ffs->fs.sb1->fsize_b);
 	ffs->ffsbsize_b = gets32(fs, ffs->fs.sb1->bsize_b);
 	ffs->ffsbsize_f = gets32(fs, ffs->fs.sb1->bsize_frag);
+	ffs->groups_count = gets32(fs, ffs->fs.sb1->cg_num);
     }
 
+
+
     fs->first_block = 0;
     fs->last_block = fs->block_count - 1;
     fs->dev_bsize = FFS_DEV_BSIZE;
@@ -1805,18 +1813,13 @@ ffs_open(IMG_INFO * img_info, unsigned char ftype, uint8_t test)
     }
 
 
-
     if (fs->ftype == FFS_2) {
 	fs->inum_count =
-	    gets32(fs, ffs->fs.sb2->cg_num) * gets32(fs,
-						     ffs->fs.sb2->
-						     cg_inode_num);
+	    ffs->groups_count * gets32(fs, ffs->fs.sb2->cg_inode_num);
     }
     else {
 	fs->inum_count =
-	    gets32(fs, ffs->fs.sb1->cg_num) * gets32(fs,
-						     ffs->fs.sb1->
-						     cg_inode_num);
+	    ffs->groups_count * gets32(fs, ffs->fs.sb1->cg_inode_num);
     }
 
     fs->root_inum = FFS_ROOTINO;
@@ -1827,9 +1830,13 @@ ffs_open(IMG_INFO * img_info, unsigned char ftype, uint8_t test)
     /*
      * Other initialization: caches, callbacks.
      */
-    ffs->cg_buf = NULL;
-    ffs->cg_num = -1;
+    ffs->grp_buf = NULL;
+    ffs->grp_num = 0xffffffff;
+
     ffs->dino_buf = NULL;
+    ffs->dino_inum = 0xffffffff;
+
+    ffs->itbl_buf = NULL;
 
     fs->inode_walk = ffs_inode_walk;
     fs->block_walk = ffs_block_walk;
@@ -1845,11 +1852,6 @@ ffs_open(IMG_INFO * img_info, unsigned char ftype, uint8_t test)
     fs->jopen = ffs_jopen;
     fs->journ_inum = 0;
 
-    /* allocate the dinode buffer */
-    if (fs->ftype == FFS_2)
-	ffs->dinode = (char *) mymalloc(sizeof(ffs_inode2));
-    else
-	ffs->dinode = (char *) mymalloc(sizeof(ffs_inode1));
 
 
     /*
@@ -1859,8 +1861,7 @@ ffs_open(IMG_INFO * img_info, unsigned char ftype, uint8_t test)
 	fprintf(logfp,
 		"inodes %" PRIuINUM " root ino %" PRIuINUM " cyl groups %"
 		PRId32 " blocks %" PRIuDADDR "\n", fs->inum_count,
-		fs->root_inum, gets32(fs, ffs->fs.sb1->cg_num),
-		fs->block_count);
+		fs->root_inum, ffs->groups_count, fs->block_count);
 
     return (fs);
 }
