@@ -1,7 +1,7 @@
 /*
 ** The Sleuth Kit 
 **
-** $Date: 2005/01/17 22:40:16 $
+** $Date: 2005/06/13 19:27:16 $
 **
 ** Brian Carrier [carrier@sleuthkit.org]
 ** Copyright (c) 2003-2005 Brian Carrier.  All rights reserved
@@ -457,13 +457,19 @@ typedef struct {
 	ffs_sb1 *sb1;		/* super block buffer */
 	ffs_sb2 *sb2;		/* super block buffer */
     } fs;
-    char *dinode;		/* disk inode cache */
-    INUM_T inum;		/* address of cached disk inode */
-    DATA_BUF *cg_buf;		/* cylinder block buffer */
-    FFS_GRPNUM_T cg_num;	/* number of cached cyl */
-    DATA_BUF *dino_buf;		/* inode block buffer */
-    int ffsbsize_f;		/* num of frags in an FFS block */
-    int ffsbsize_b;		/* size of an FFS block in bytes */
+
+    char *dino_buf;		/* cached disk inode */
+    INUM_T dino_inum;		/* address of cached disk inode */
+
+    DATA_BUF *itbl_buf;		/* cached inode block buffer */
+
+    DATA_BUF *grp_buf;		/* Cached cylinder group buffer */
+    FFS_GRPNUM_T grp_num;	/* number of cached cyl */
+
+    FFS_GRPNUM_T groups_count;	/* nr of descriptor group blocks */
+
+    unsigned int ffsbsize_f;	/* num of frags in an FFS block */
+    unsigned int ffsbsize_b;	/* size of an FFS block in bytes */
 } FFS_INFO;
 
 
