@@ -2,7 +2,7 @@
 ** fs_data
 ** The Sleuth Kit 
 **
-** $Date: 2005/01/17 22:40:16 $
+** $Date: 2005/06/13 19:27:16 $
 **
 ** Brian Carrier [carrier@sleuthkit.org]
 ** Copyright (c) 2003-2005 Brian Carrier.  All rights reserved 
@@ -280,7 +280,7 @@ fs_data_put_name(FS_DATA * data, char *name)
  */
 FS_DATA *
 fs_data_put_str(FS_DATA * data_head, char *name, uint32_t type,
-		uint16_t id, void *addr, int len)
+		uint16_t id, void *addr, unsigned int len)
 {
 
     FS_DATA *data;
