@@ -1,7 +1,7 @@
 /*
 ** The Sleuth Kit
 **
-** $Date: 2005/01/17 22:40:16 $
+** $Date: 2005/06/13 19:27:16 $
 **
 */
 
@@ -14,7 +14,7 @@ extern FS_DATA *fs_data_getnew_attr(FS_DATA *, uint8_t);
 extern void fs_data_clear_list(FS_DATA *);
 
 extern FS_DATA *fs_data_put_str(FS_DATA *, char *, uint32_t, uint16_t,
-				void *, int);
+				void *, unsigned int);
 
 extern FS_DATA *fs_data_put_run(FS_DATA *, DADDR_T, OFF_T,
 				FS_DATA_RUN *, char *, uint32_t, uint16_t,
