@@ -1,7 +1,7 @@
 /* 
  * The Sleuth Kit
  *
- * $Date: 2005/01/17 22:40:16 $
+ * $Date: 2005/06/13 19:27:16 $
  * 
  * Brian Carrier [carrier@sleuthkit.org]
  * Copyright (c) 2005 Brian Carrier.  All rights reserved
@@ -15,7 +15,7 @@
 */
 
 uint8_t
-load_file_action(FS_INFO * fs, DADDR_T addr, char *buf, int size,
+load_file_action(FS_INFO * fs, DADDR_T addr, char *buf, unsigned int size,
 		 int flags, void *ptr)
 {
     FS_LOAD_FILE *buf1 = (FS_LOAD_FILE *) ptr;
