@@ -2,7 +2,7 @@
 ** fs_open
 ** The Sleuth Kit 
 **
-** $Date: 2005/03/06 19:48:15 $
+** $Date: 2005/07/08 17:20:10 $
 **
 ** Brian Carrier [carrier@sleuthkit.org]
 ** Copyright (c) 2003-2005 Brian Carrier.  All rights reserved 
@@ -70,14 +70,28 @@ fs_open(IMG_INFO * img_info, const char *type)
 	}
 
 	if ((fs_info = ffs_open(img_info, FFSAUTO, 1)) != NULL) {
-	    if (set != NULL) {
+	    if (set == NULL) {
+		set = "UFS";
+		fs_set = fs_info;
+	    }
+	    else {
 		fs_set->close(fs_set);
 		fs_info->close(fs_info);
 		error("Cannot determine if UFS or %s", set);
 	    }
-	    fs_set = fs_info;
 	}
 
+	/*
+	   if ((fs_info = iso9660_open(img_info, ISO9660, 1)) != NULL) {
+	   if (set != NULL) {
+	   fs_set->close(fs_set);
+	   fs_info->close(fs_info);
+	   error("Cannot determine if ISO9660 or %s", set);
+	   }
+	   fs_set = fs_info;
+	   }
+	 */
+
 	if (fs_set != NULL)
 	    return fs_set;
 	else {
@@ -98,6 +112,10 @@ fs_open(IMG_INFO * img_info, const char *type)
 	    return fatfs_open(img_info, ftype, 0);
 	case NTFS_TYPE:
 	    return ntfs_open(img_info, ftype, 0);
+/*
+	case ISO9660_TYPE:
+	    return iso9660_open(img_info, ftype, 0);
+ */
 	case RAWFS_TYPE:
 	    return rawfs_open(img_info, ftype);
 	case SWAPFS_TYPE:
@@ -106,7 +124,7 @@ fs_open(IMG_INFO * img_info, const char *type)
 	default:
 	    fprintf(stderr, "unknown filesystem type: %s\n", type);
 	    fprintf(stderr, "known types:\n");
-	    fs_print_types();
+	    fs_print_types(stderr);
 	    exit(1);
 	}
     }
