@@ -2,7 +2,7 @@
 ** fs_tools
 ** The Sleuth Kit 
 **
-** $Date: 2005/03/25 03:12:34 $
+** $Date: 2005/07/08 17:20:10 $
 **
 ** Brian Carrier [carrier@sleuthkit.org]
 ** Copyright (c) 2003-2005 Brian Carrier.  All rights reserved
@@ -47,15 +47,17 @@
 extern FILE *logfp;
 
 
-#ifndef NBBY			/* NIH */
+#ifndef NBBY
 #define NBBY 8
 #endif
 
-#ifndef isset			/* NIH */
-#define isset(a,i)	((a)[(i)/NBBY] & (1<<((i)%NBBY)))
+#ifndef isset
+#define isset(a,i)	(((uint8_t *)(a))[(i)/NBBY] &  (1<<((i)%NBBY)))
 #endif
 
-
+#ifndef setbit
+#define setbit(a,i)     (((uint8_t *)(a))[(i)/NBBY] |= (1<<((i)%NBBY)))
+#endif
 
 
 typedef struct FS_INFO FS_INFO;
@@ -73,13 +75,12 @@ typedef struct FS_JENTRY FS_JENTRY;
 #define WALK_STOP	0x1
 
 /* walk action functions */
-typedef uint8_t(*FS_INODE_WALK_FN) (FS_INFO *, INUM_T, FS_INODE *, int,
-				    void *);
+typedef uint8_t(*FS_INODE_WALK_FN) (FS_INFO *, FS_INODE *, int, void *);
 typedef uint8_t(*FS_BLOCK_WALK_FN) (FS_INFO *, DADDR_T, char *, int,
 				    void *);
 typedef uint8_t(*FS_DENT_WALK_FN) (FS_INFO *, FS_DENT *, int, void *);
-typedef uint8_t(*FS_FILE_WALK_FN) (FS_INFO *, DADDR_T, char *, int, int,
-				   void *);
+typedef uint8_t(*FS_FILE_WALK_FN) (FS_INFO *, DADDR_T, char *,
+				   unsigned int, int, void *);
 
 typedef uint8_t(*FS_JBLK_WALK_FN) (FS_INFO *, char *, int, void *);
 typedef uint8_t(*FS_JENTRY_WALK_FN) (FS_INFO *, FS_JENTRY *, int, void *);
@@ -95,8 +96,8 @@ typedef struct {
     size_t left;
 } FS_LOAD_FILE;
 
-extern uint8_t load_file_action(FS_INFO *, DADDR_T, char *, int, int,
-				void *);
+extern uint8_t load_file_action(FS_INFO *, DADDR_T, char *, unsigned int,
+				int, void *);
 
 /***************************************************************
  * FS_INFO: Allocated when an image is opened
@@ -114,8 +115,8 @@ struct FS_INFO {
     DADDR_T block_count;	/* number of blocks */
     DADDR_T first_block;	/* in case start at 1 */
     DADDR_T last_block;		/* in case start at 1 */
-    int block_size;		/* block size in bytes */
-    int dev_bsize;		/* size of device blocks */
+    unsigned int block_size;	/* block size in bytes */
+    unsigned int dev_bsize;	/* size of device blocks */
 
     /* Journal */
     INUM_T journ_inum;		/* Inode of journal */
@@ -224,7 +225,7 @@ struct FS_DATA {
     FS_DATA *next;
     uint8_t flags;
     char *name;			/* name of data (if available) */
-    int nsize;			/* number of allocated bytes for name */
+    unsigned int nsize;		/* number of allocated bytes for name */
     uint32_t type;		/* type of attribute */
     uint16_t id;		/* id of attr, used when duplicate types */
 
@@ -257,6 +258,7 @@ struct FS_NAME {
 };
 
 struct FS_INODE {
+    INUM_T addr;		/* Address of meta data structure */
     mode_t mode;		/* type and permission */
     int nlink;			/* link count */
     OFF_T size;			/* file size */
@@ -293,8 +295,6 @@ extern void fs_inode_free(FS_INODE *);
  */
 struct FS_DENT {
     INUM_T inode;		/* inode number */
-// @@@ This should be locally defined in each file system 
-    uint16_t reclen;		/* actual length of record based on name len */
     uint8_t ent_type;		/* dir, file etc FS_DENT_??? */
     uint16_t namlen;		/* length of actual name */
 // @@@ This should be locally defined in each file system 
@@ -416,6 +416,7 @@ extern FS_INFO *ffs_open(IMG_INFO *, unsigned char ftype, uint8_t);
 extern FS_INFO *ntfs_open(IMG_INFO *, unsigned char ftype, uint8_t);
 extern FS_INFO *rawfs_open(IMG_INFO *, unsigned char ftype);
 extern FS_INFO *swapfs_open(IMG_INFO *, unsigned char ftype);
+extern FS_INFO *iso9660_open(IMG_INFO *, unsigned char, uint8_t);
 
 
 // Endian macros - actual functions in misc/
