@@ -2,7 +2,7 @@
 ** fs_types
 ** The Sleuth Kit 
 **
-** $Date: 2005/01/17 22:40:16 $
+** $Date: 2005/06/13 19:27:17 $
 **
 ** Brian Carrier [carrier@sleuthkit.org]
 ** Copyright (c) 2003-2005 Brian Carrier.  All rights reserved
@@ -15,7 +15,7 @@
 #define _FS_TYPES_H
 
 extern char fs_parse_type(const char *);
-extern void fs_print_types();
+extern void fs_print_types(FILE *);
 extern char *fs_get_type(char);
 extern void print_version();
 
@@ -60,4 +60,8 @@ extern void print_version();
 #define	RAWFS_TYPE		0x60
 #define	RAW				0x60
 
+
+#define ISO9660_TYPE		0x70
+#define ISO9660			0x70
+
 #endif
