@@ -2,7 +2,7 @@
 ** icat 
 ** The Sleuth Kit 
 **
-** $Date: 2005/03/25 03:12:34 $
+** $Date: 2005/05/18 00:41:57 $
 **
 ** Brian Carrier [carrier@sleuthkit.org]
 ** Copyright (c) 2003-2005 Brian Carrier.  All rights reserved 
@@ -62,7 +62,9 @@ main(int argc, char **argv)
     char *fstype = NULL;
     char *imgtype = NULL, *imgoff = NULL, *dash;
     IMG_INFO *img;
-    int type = 0, id = 0, id_used = 0;
+    uint32_t type = 0;
+    uint16_t id = 0;
+    int id_used = 0;
 
     progname = argv[0];
 
@@ -129,7 +131,7 @@ main(int argc, char **argv)
 	    *dash2 = '\0';
 	    dash2++;
 
-	    id = strtoull(dash2, &cp, 0);
+	    id = (uint16_t) strtoul(dash2, &cp, 0);
 	    id_used = 1;
 	    if (*cp || cp == dash2) {
 		fprintf(stderr, "Invalid id in inode\n");
@@ -143,7 +145,7 @@ main(int argc, char **argv)
 	    usage();
 	}
 
-	type = strtoull(dash, &cp, 0);
+	type = strtoul(dash, &cp, 0);
 	if (*cp || cp == dash) {
 	    fprintf(stderr, "Invalid type in inode\n");
 	    usage();
