@@ -2,7 +2,7 @@
 ** ifind (inode find)
 ** The Sleuth Kit
 **
-** $Date: 2005/03/25 03:12:34 $
+** $Date: 2005/07/08 17:20:10 $
 **
 ** Given an image  and block number, identify which inode it is used by
 ** 
@@ -67,18 +67,17 @@ static INUM_T parinode = 0;
 /* dent call back for finding unallocated files based on parent directory
  */
 static uint8_t
-ifind_par_act(FS_INFO * fs, INUM_T inum, FS_INODE * fsi, int flags,
-	      void *ptr)
+ifind_par_act(FS_INFO * fs, FS_INODE * fs_inode, int flags, void *ptr)
 {
     FS_NAME *fs_name;
 
     /* go through each file name structure */
-    fs_name = fsi->name;
+    fs_name = fs_inode->name;
     while (fs_name) {
 	if (fs_name->par_inode == parinode) {
 	    /* Fill in teh basics of the fs_dent entry */
-	    fs_dent->fsi = fsi;
-	    fs_dent->inode = inum;
+	    fs_dent->fsi = fs_inode;
+	    fs_dent->inode = fs_inode->addr;
 	    strncpy(fs_dent->name, fs_name->name, fs_dent->maxnamlen);
 	    if (localflags & IFIND_PAR_LONG) {
 		fs_dent_print_long(stdout, fs_dent, FS_FLAG_NAME_UNALLOC,
@@ -211,7 +210,7 @@ ifind_path_act(FS_INFO * fs, FS_DENT * fs_dent, int flags, void *ptr)
 	 * paren at the end */
 	else if (fs_dent->name[strlen(fs_dent->name) - 1] == ')') {
 	    char *sh_ptr;
-	    int long_len = 0, sh_len = 0;
+	    unsigned int long_len = 0, sh_len = 0;
 
 	    /* Get the beginning of the short name */
 	    sh_ptr = strrchr(fs_dent->name, '(');
@@ -365,7 +364,7 @@ static uint16_t curid;
  */
 static uint8_t
 ifind_data_file_act(FS_INFO * fs, DADDR_T addr, char *buf,
-		    int size, int flags, void *ptr)
+		    unsigned int size, int flags, void *ptr)
 {
     /* Drop references to block zero (sparse)
      * This becomes an issue with fragments and looking for fragments
@@ -395,7 +394,7 @@ ifind_data_file_act(FS_INFO * fs, DADDR_T addr, char *buf,
  */
 static uint8_t
 ifind_data_file_ntfs_act(FS_INFO * fs, DADDR_T addr, char *buf,
-			 int size, int flags, void *ptr)
+			 unsigned int size, int flags, void *ptr)
 {
     if (addr == block) {
 	printf("%" PRIuINUM "-%" PRIu32 "-%" PRIu16 "\n", curinode,
@@ -418,8 +417,7 @@ ifind_data_file_ntfs_act(FS_INFO * fs, DADDR_T addr, char *buf,
 ** Callback action for inode_walk
 */
 static uint8_t
-ifind_data_act(FS_INFO * fs, INUM_T inum, FS_INODE * fsi, int flags,
-	       void *ptr)
+ifind_data_act(FS_INFO * fs, FS_INODE * fs_inode, int flags, void *ptr)
 {
     int file_flags = (FS_FLAG_FILE_AONLY | FS_FLAG_FILE_NOABORT);
 
@@ -427,19 +425,19 @@ ifind_data_act(FS_INFO * fs, INUM_T inum, FS_INODE * fsi, int flags,
     if (flags & FS_FLAG_META_UNALLOC)
 	file_flags |= FS_FLAG_FILE_RECOVER;
 
-    curinode = inum;
+    curinode = fs_inode->addr;
 
     /* NT Specific Stuff: search all ADS */
     if ((fs->ftype & FSMASK) == NTFS_TYPE) {
-	FS_DATA *data = fsi->attr;
+	FS_DATA *data = fs_inode->attr;
 
 	file_flags |= FS_FLAG_FILE_SLACK;
 	while ((data) && (data->flags & FS_DATA_INUSE)) {
 	    curtype = data->type;
 	    curid = data->id;
 	    if (data->flags & FS_DATA_NONRES) {
-		fs->file_walk(fs, fsi, data->type, data->id, file_flags,
-			      ifind_data_file_ntfs_act, ptr);
+		fs->file_walk(fs, fs_inode, data->type, data->id,
+			      file_flags, ifind_data_file_ntfs_act, ptr);
 	    }
 	    data = data->next;
 	}
@@ -447,7 +445,8 @@ ifind_data_act(FS_INFO * fs, INUM_T inum, FS_INODE * fsi, int flags,
     }
     else if ((fs->ftype & FSMASK) == FATFS_TYPE) {
 	file_flags |= (FS_FLAG_FILE_SLACK | FS_FLAG_FILE_NOID);
-	fs->file_walk(fs, fsi, 0, 0, file_flags, ifind_data_file_act, ptr);
+	fs->file_walk(fs, fs_inode, 0, 0, file_flags, ifind_data_file_act,
+		      ptr);
     }
     /* UNIX do not need the SLACK flag because they use fragments - if the
      * SLACK flag exists then any unused fragments in a block will be 
@@ -455,7 +454,8 @@ ifind_data_act(FS_INFO * fs, INUM_T inum, FS_INODE * fsi, int flags,
      */
     else {
 	file_flags |= (FS_FLAG_FILE_NOID);
-	fs->file_walk(fs, fsi, 0, 0, file_flags, ifind_data_file_act, ptr);
+	fs->file_walk(fs, fs_inode, 0, 0, file_flags, ifind_data_file_act,
+		      ptr);
     }
 
     return WALK_CONT;
