@@ -1,7 +1,7 @@
 /*
 ** The Sleuth Kit 
 **
-** $Date: 2005/03/25 03:12:34 $
+** $Date: 2005/07/08 17:20:10 $
 **
 ** Brian Carrier [carrier@sleuthkit.org]
 ** Copyright (c) 2003-2005 Brian Carrier.  All rights reserved 
@@ -54,11 +54,12 @@ static void
 usage()
 {
     fprintf(stderr,
-	    "usage: %s [-emrvV] [-aAlLzZ] [-f fstype] [-i imgtype] [-o imgoffset] [-s seconds] image [images] [inum[-end]]\n",
+	    "usage: %s [-eOmrvV] [-aAlLzZ] [-f fstype] [-i imgtype] [-o imgoffset] [-s seconds] image [images] [inum[-end]]\n",
 	    progname);
 
     fprintf(stderr, "\t-e: Display all inodes\n");
-//      fprintf(stderr, "\t-o: Display inodes that are removed, but sill open\n");
+    fprintf(stderr,
+	    "\t-O: Display inodes that are removed, but sill open (was -o)\n");
     fprintf(stderr,
 	    "\t-m: Display output in the mactime format (replaces ils2mac from TCT)\n");
     fprintf(stderr, "\t-r: Display removed inodes (default)\n");
@@ -103,14 +104,13 @@ main(int argc, char **argv)
     char *image = NULL;
     int32_t sec_skew = 0;
 
-
     progname = argv[0];
 
     /*
      * Provide convenience options for the most commonly selected feature
      * combinations.
      */
-    while ((ch = getopt(argc, argv, "aAef:i:lLo:mrs:vVzZ")) > 0) {
+    while ((ch = getopt(argc, argv, "aAef:i:lLo:Omrs:vVzZ")) > 0) {
 	switch (ch) {
 	case '?':
 	default:
@@ -125,18 +125,16 @@ main(int argc, char **argv)
 	case 'i':
 	    imgtype = optarg;
 	    break;
-
-
 	case 'm':
 	    argflags |= ILS_MAC;
 	    break;
 	case 'o':
 	    imgoff = optarg;
 	    break;
-//      case 'o':
-//              flags |= (FS_FLAG_META_ALLOC | FS_FLAG_META_UNLINK);
-//              argflags |= ILS_OPEN;
-//          break;
+	case 'O':
+	    flags |= (FS_FLAG_META_ALLOC | FS_FLAG_META_UNLINK);
+	    argflags |= ILS_OPEN;
+	    break;
 	case 'r':
 	    argflags |= ILS_REM;
 	    break;
