@@ -1,7 +1,7 @@
 /*
 ** The Sleuth Kit 
 **
-** $Date: 2005/03/25 03:12:34 $
+** $Date: 2005/06/15 15:58:03 $
 **
 ** Brian Carrier [carrier@sleuthkit.org]
 ** Copyright (c) 2003-2005 Brian Carrier.  All rights reserved 
@@ -93,8 +93,7 @@ print_header_mac()
 /* print_inode - list generic inode */
 
 static uint8_t
-ils_act(FS_INFO * fs, INUM_T inum, FS_INODE * fs_inode, int flags,
-	void *ptr)
+ils_act(FS_INFO * fs, FS_INODE * fs_inode, int flags, void *ptr)
 {
 
     if (sec_skew != 0) {
@@ -103,7 +102,7 @@ ils_act(FS_INFO * fs, INUM_T inum, FS_INODE * fs_inode, int flags,
 	fs_inode->ctime -= sec_skew;
     }
     printf("%" PRIuINUM "|%c|%d|%d|%lu|%lu|%lu",
-	   inum, (flags & FS_FLAG_META_ALLOC) ? 'a' : 'f',
+	   fs_inode->addr, (flags & FS_FLAG_META_ALLOC) ? 'a' : 'f',
 	   (int) fs_inode->uid, (int) fs_inode->gid,
 	   (ULONG) fs_inode->mtime, (ULONG) fs_inode->atime,
 	   (ULONG) fs_inode->ctime);
@@ -129,8 +128,7 @@ ils_act(FS_INFO * fs, INUM_T inum, FS_INODE * fs_inode, int flags,
  */
 
 static uint8_t
-ils_mac_act(FS_INFO * fs, INUM_T inum, FS_INODE * fs_inode, int flags,
-	    void *ptr)
+ils_mac_act(FS_INFO * fs, FS_INODE * fs_inode, int flags, void *ptr)
 {
     char ls[12];
 
@@ -138,8 +136,8 @@ ils_mac_act(FS_INFO * fs, INUM_T inum, FS_INODE * fs_inode, int flags,
     printf("0|<%s-%s%s%s-%" PRIuINUM ">|0|%" PRIuINUM "|%d|", image,
 	   (fs_inode->name) ? fs_inode->name->name : "",
 	   (fs_inode->name) ? "-" : "",
-	   (flags & FS_FLAG_META_ALLOC) ? "alive" : "dead", inum,
-	   inum, (int) fs_inode->mode);
+	   (flags & FS_FLAG_META_ALLOC) ? "alive" : "dead", fs_inode->addr,
+	   fs_inode->addr, (int) fs_inode->mode);
 
     /* Print the "ls" mode in ascii format */
     make_ls(fs_inode->mode, ls, 12);
