@@ -2,7 +2,7 @@
 ** jls
 ** The Sleuth Kit 
 **
-** $Date: 2005/01/17 22:40:16 $
+** $Date: 2005/06/13 19:27:17 $
 **
 ** Brian Carrier [carrier@sleuthkit.org]
 ** Copyright (c) 2004-2005 Brian Carrier.  All rights reserved
@@ -177,7 +177,7 @@ main(int argc, char **argv)
     }
 
     fs->jopen(fs, inum);
-    fs->jentry_walk(fs, 0, 0, "");
+    fs->jentry_walk(fs, 0, 0, NULL);
 
     fs->close(fs);
     img->close(img);
