@@ -2,7 +2,7 @@
 ** ntfs
 ** The Sleuth Kit 
 **
-** $Date: 2005/03/04 05:41:26 $
+** $Date: 2005/07/08 17:20:10 $
 **
 ** Content and meta data layer support for the NTFS file system
 **
@@ -66,8 +66,6 @@ static void
  * that the sanity checks still work.
  */
 
-// set to 1 when $MFT is itself being loaded
-static uint8_t loading_the_MFT = 0;
 
 /**********************************************************************
  *
@@ -114,10 +112,10 @@ nt2unixtime(uint64_t ntdate)
  * structure
  */
 static void
-ntfs_mft_lookup(NTFS_INFO * ntfs, ntfs_mft * mft, INUM_T mftnum)
+ntfs_dinode_lookup(NTFS_INFO * ntfs, ntfs_mft * mft, INUM_T mftnum)
 {
     OFF_T mftaddr_b, mftaddr2_b, offset;
-    int mftaddr_len = 0;
+    unsigned int mftaddr_len = 0;
     int i;
     FS_INFO *fs = (FS_INFO *) & ntfs->fs_info;
     FS_DATA_RUN *data_run;
@@ -136,7 +134,8 @@ ntfs_mft_lookup(NTFS_INFO * ntfs, ntfs_mft * mft, INUM_T mftnum)
 
     if (verbose)
 	fprintf(logfp,
-		"ntfs_mft_lookup: Processing MFT %" PRIuINUM "\n", mftnum);
+		"ntfs_dinode_lookup: Processing MFT %" PRIuINUM "\n",
+		mftnum);
 
     /* If mft_data (the cached $Data attribute of $MFT) is not there yet, 
      * then we have not started to load $MFT yet.  In that case, we will
@@ -186,9 +185,10 @@ ntfs_mft_lookup(NTFS_INFO * ntfs, ntfs_mft * mft, INUM_T mftnum)
 
 		if (verbose)
 		    fprintf(logfp,
-			    "ntfs_mft_lookup: Found in offset: %" PRIuDADDR
-			    "  size: %" PRIuDADDR " at offset: %" PRIuOFF
-			    "\n", data_run->addr, data_run->len, offset);
+			    "ntfs_dinode_lookup: Found in offset: %"
+			    PRIuDADDR "  size: %" PRIuDADDR " at offset: %"
+			    PRIuOFF "\n", data_run->addr, data_run->len,
+			    offset);
 
 		/* special case where the MFT entry crosses
 		 * a run (only happens when cluster size is 512-bytes
@@ -198,7 +198,7 @@ ntfs_mft_lookup(NTFS_INFO * ntfs, ntfs_mft * mft, INUM_T mftnum)
 
 		    if (verbose)
 			fprintf(logfp,
-				"ntfs_mft_lookup: Entry crosses run border\n");
+				"ntfs_dinode_lookup: Entry crosses run border\n");
 
 		    if (data_run->next == NULL)
 			error
@@ -215,8 +215,8 @@ ntfs_mft_lookup(NTFS_INFO * ntfs, ntfs_mft * mft, INUM_T mftnum)
 		mftaddr_b = data_run->addr * ntfs->csize_b + offset;
 		if (verbose)
 		    fprintf(logfp,
-			    "ntfs_mft_lookup: Entry address at: %" PRIuOFF
-			    "\n", mftaddr_b);
+			    "ntfs_dinode_lookup: Entry address at: %"
+			    PRIuOFF "\n", mftaddr_b);
 		break;
 	    }
 
@@ -235,8 +235,9 @@ ntfs_mft_lookup(NTFS_INFO * ntfs, ntfs_mft * mft, INUM_T mftnum)
 	/* read the first part into mft */
 	if (fs_read_random(&ntfs->fs_info, (char *) mft,
 			   mftaddr_len, mftaddr_b) != mftaddr_len) {
-	    error("ntfs_mft_lookup: Error reading MFT Entry (part 1) at %"
-		  PRIuOFF ": %m", mftaddr_b);
+	    error
+		("ntfs_dinode_lookup: Error reading MFT Entry (part 1) at %"
+		 PRIuOFF ": %m", mftaddr_b);
 	}
 
 	/* read the second part into mft */
@@ -244,8 +245,9 @@ ntfs_mft_lookup(NTFS_INFO * ntfs, ntfs_mft * mft, INUM_T mftnum)
 	    (&ntfs->fs_info, (char *) ((uintptr_t) mft + mftaddr_len),
 	     ntfs->mft_rsize_b - mftaddr_len,
 	     mftaddr2_b) != ntfs->mft_rsize_b - mftaddr_len) {
-	    error("ntfs_mft_lookup: Error reading MFT Entry (part 2) at %"
-		  PRIuOFF ": %m", mftaddr2_b);
+	    error
+		("ntfs_dinode_lookup: Error reading MFT Entry (part 2) at %"
+		 PRIuOFF ": %m", mftaddr2_b);
 	}
     }
     else {
@@ -253,8 +255,8 @@ ntfs_mft_lookup(NTFS_INFO * ntfs, ntfs_mft * mft, INUM_T mftnum)
 	if (fs_read_random(&ntfs->fs_info, (char *) mft,
 			   ntfs->mft_rsize_b,
 			   mftaddr_b) != ntfs->mft_rsize_b) {
-	    error("ntfs_mft_lookup: Error reading MFT Entry at %" PRIuOFF
-		  ": %m", mftaddr_b);
+	    error("ntfs_dinode_lookup: Error reading MFT Entry at %"
+		  PRIuOFF ": %m", mftaddr_b);
 	}
     }
 
@@ -298,10 +300,10 @@ ntfs_mft_lookup(NTFS_INFO * ntfs, ntfs_mft * mft, INUM_T mftnum)
 
     /* cycle through each sector */
     for (i = 1; i < getu16(fs, mft->upd_cnt); i++) {
+	uint8_t *new_val, *old_val;
 
 	/* The offset into the buffer of the value to analyze */
 	size_t offset = i * ntfs->ssize_b - 2;
-	uint8_t *new, *old;
 
 	/* get the current sequence value */
 	uint16_t cur_seq = getu16(fs, (uintptr_t) mft + offset);
@@ -317,17 +319,18 @@ ntfs_mft_lookup(NTFS_INFO * ntfs, ntfs_mft * mft, INUM_T mftnum)
 		 sig_seq, cur_seq, cur_repl);
 	}
 
-	new = &upd->upd_seq + (i - 1) * 2;
-	old = (uint8_t *) ((uintptr_t) mft + offset);
+	new_val = &upd->upd_seq + (i - 1) * 2;
+	old_val = (uint8_t *) ((uintptr_t) mft + offset);
 
 	if (verbose)
 	    fprintf(logfp,
-		    "ntfs_mft_lookup: upd_seq %i   Replacing: %.4" PRIx16
-		    "   With: %.4" PRIx16 "\n", i, getu16(fs, old),
-		    getu16(fs, new));
+		    "ntfs_dinode_lookup: upd_seq %i   Replacing: %.4"
+		    PRIx16 "   With: %.4" PRIx16 "\n", i, getu16(fs,
+								 old_val),
+		    getu16(fs, new_val));
 
-	*old++ = *new++;
-	*old = *new;
+	*old_val++ = *new_val++;
+	*old_val = *new_val;
     }
 
     return;
@@ -349,7 +352,7 @@ is_clust_alloc(NTFS_INFO * ntfs, DADDR_T addr)
      * is allocated.  This should only be needed when we are
      * dealing with an attribute list ... 
      */
-    if (loading_the_MFT == 1)
+    if (ntfs->loading_the_MFT == 1)
 	return 1;
     else if (ntfs->bmap == NULL)
 	error("Bitmap pointer is null: %" PRIuDADDR "\n", addr);
@@ -426,7 +429,7 @@ ntfs_make_data_run(NTFS_INFO * ntfs, ntfs_runlist * runlist,
 {
     ntfs_runlist *run;
     FS_DATA_RUN *data_run, *data_run_head = NULL, *data_run_prev = NULL;
-    int i, idx;
+    unsigned int i, idx;
     DADDR_T prev_addr = 0;
 
     run = runlist;
@@ -436,10 +439,11 @@ ntfs_make_data_run(NTFS_INFO * ntfs, ntfs_runlist * runlist,
 	*runlen = 0;
 
     /* Cycle through each run in the runlist 
-     * We got until we find an entry for a run with no length
-     * An entry with offset of 0 is for sparse runs
+     * We go until we find an entry with no length
+     * An entry with offset of 0 is for a sparse run
      */
     while (NTFS_RUNL_LENSZ(run) != 0) {
+	int64_t offset = 0;
 
 	/* allocate a new fs_data_run */
 	data_run = fs_data_run_alloc();
@@ -475,25 +479,26 @@ ntfs_make_data_run(NTFS_INFO * ntfs, ntfs_runlist * runlist,
 
 	/* Get the address of this run */
 	for (i = 0, data_run->addr = 0; i < NTFS_RUNL_OFFSZ(run); i++) {
-	    data_run->addr |= (run->buf[idx++] << (i * 8));
+	    //data_run->addr |= (run->buf[idx++] << (i * 8));
+	    offset |= (run->buf[idx++] << (i * 8));
 	    if (verbose)
 		fprintf(logfp,
 			"ntfs_make_data_run: Off idx: %i cur: %" PRIu8
 			" (%" PRIx8 ") tot: %" PRIuDADDR " (%" PRIxDADDR
 			")\n", i, run->buf[idx - 1], run->buf[idx - 1],
-			data_run->addr, data_run->addr);
+			offset, offset);
 	}
 
 	/* offset value is signed so extend it to 64-bits */
 	if ((int8_t) run->buf[idx - 1] < 0) {
-	    for (; i < sizeof(data_run->addr); i++)
-		data_run->addr |= (int64_t) ((int64_t) 0xff << (i * 8));
+	    for (; i < sizeof(offset); i++)
+		offset |= (int64_t) ((int64_t) 0xff << (i * 8));
 	}
 
 	if (verbose)
 	    fprintf(logfp,
 		    "ntfs_make_data_run: Signed offset: %" PRIdDADDR
-		    " Previous address: %" PRIdDADDR "\n", data_run->addr,
+		    " Previous address: %" PRIdDADDR "\n", offset,
 		    prev_addr);
 
 
@@ -506,8 +511,8 @@ ntfs_make_data_run(NTFS_INFO * ntfs, ntfs_runlist * runlist,
 	 * Change the check now to not limit to NT, but make sure
 	 * that it is the first run
 	 */
-	if (((data_run->addr == -1) && (prev_addr == 0)) ||
-	    ((data_run->addr == -1) && (ntfs->ver == NTFS_VINFO_NT))) {
+	if (((offset == -1) && (prev_addr == 0)) ||
+	    ((offset == -1) && (ntfs->ver == NTFS_VINFO_NT))) {
 	    data_run->flags |= FS_DATA_SPARSE;
 	    data_run->addr = 0;
 	    if (verbose)
@@ -524,8 +529,8 @@ ntfs_make_data_run(NTFS_INFO * ntfs, ntfs_runlist * runlist,
 	 * For sparse files the next run will have its offset relative 
 	 * to the current "prev_addr" so skip that code
 	 */
-	else if ((data_run->addr) || (ntfs->mnum == NTFS_MFT_BOOT)) {
-	    data_run->addr += prev_addr;
+	else if ((offset) || (ntfs->mnum == NTFS_MFT_BOOT)) {
+	    data_run->addr = prev_addr + offset;
 	    prev_addr = data_run->addr;
 	}
 	else {
@@ -574,12 +579,16 @@ ntfs_make_data_run(NTFS_INFO * ntfs, ntfs_runlist * runlist,
  * No notion of META
  */
 void
-ntfs_data_walk(NTFS_INFO * ntfs, FS_DATA * fs_data,
+ntfs_data_walk(NTFS_INFO * ntfs, INUM_T inum, FS_DATA * fs_data,
 	       int flags, FS_FILE_WALK_FN action, void *ptr)
 {
     FS_INFO *fs = (FS_INFO *) & ntfs->fs_info;
     int myflags;
 
+    if (verbose)
+	fprintf(logfp, "ntfs_data_walk: Processing file %" PRIuINUM "\n",
+		inum);
+
     /* Process the resident buffer 
      */
     if (fs_data->flags & FS_DATA_RES) {
@@ -598,7 +607,7 @@ ntfs_data_walk(NTFS_INFO * ntfs, FS_DATA * fs_data,
     }
     /* non-resident */
     else {
-	int a, bufsize;
+	unsigned int a, bufsize;
 	DADDR_T addr;
 	DATA_BUF *data_buf = NULL;
 	char *buf = NULL;
@@ -759,15 +768,15 @@ ntfs_proc_attrseq(NTFS_INFO * ntfs, FS_INODE * fs_inode,
 
     if (verbose)
 	fprintf(logfp,
-		"ntfs_proc_attrseq: Processing MFT %" PRIuINUM
-		" (maybe)\n", ntfs->mnum);
+		"ntfs_proc_attrseq: Processing entry %" PRIuINUM
+		"\n", fs_inode->addr);
 
     /* Cycle through the list of attributes */
     for (; ((uintptr_t) attr >= (uintptr_t) attrseq) &&
 	 ((uintptr_t) attr <= ((uintptr_t) attrseq + len)) &&
 	 (getu32(fs, attr->len) > 0 &&
 	  (getu32(fs, attr->type) != 0xffffffff));
-	 (uintptr_t) attr = (uintptr_t) attr + getu32(fs, attr->len)) {
+	 attr = (ntfs_attr *) ((uintptr_t) attr + getu32(fs, attr->len))) {
 	/* Get the type of this attribute */
 	uint32_t type = getu32(fs, attr->type);
 
@@ -1022,7 +1031,7 @@ ntfs_proc_attrseq(NTFS_INFO * ntfs, FS_INODE * fs_inode,
     }
 
     /* Are we currently in the process of loading $MFT? */
-    if (loading_the_MFT == 1) {
+    if (ntfs->loading_the_MFT == 1) {
 
 	/* If we don't even have a mini cached version, get it now 
 	 * Even if we are not done because of attribute lists, then we
@@ -1093,8 +1102,9 @@ ntfs_proc_attrlist(NTFS_INFO * ntfs, FS_INODE * fs_inode,
     uint16_t histcnt = 0;
 
     if (verbose)
-	fprintf(logfp, "ntfs_proc_attrlist: MFT %" PRIuINUM "\n",
-		ntfs->mnum);
+	fprintf(logfp,
+		"ntfs_proc_attrlist: Processing entry %" PRIuINUM "\n",
+		fs_inode->addr);
 
     /* Clear the contents of the history buffer */
     memset(hist, 0, sizeof(hist));
@@ -1109,7 +1119,7 @@ ntfs_proc_attrlist(NTFS_INFO * ntfs, FS_INODE * fs_inode,
 	mymalloc(fs_data_attrlist->size);
     endaddr = (uintptr_t) buf + fs_data_attrlist->size;
 
-    ntfs_data_walk(ntfs, fs_data_attrlist, 0, load_file_action,
+    ntfs_data_walk(ntfs, ntfs->mnum, fs_data_attrlist, 0, load_file_action,
 		   (void *) &load_file);
 
     /* this value should be zero, if not then we didn't read all of the
@@ -1185,7 +1195,7 @@ ntfs_proc_attrlist(NTFS_INFO * ntfs, FS_INODE * fs_inode,
 	    continue;
 	}
 
-	ntfs_mft_lookup(ntfs, mft, mftnum);
+	ntfs_dinode_lookup(ntfs, mft, mftnum);
 
 	/* verify that this entry refers to the original one */
 	if (getu48(fs, mft->base_ref) != ntfs->mnum) {
@@ -1225,7 +1235,7 @@ ntfs_proc_attrlist(NTFS_INFO * ntfs, FS_INODE * fs_inode,
  * Copy the MFT entry saved in ntfs->mft into the generic structure 
  */
 static void
-ntfs_copy_inode(NTFS_INFO * ntfs, FS_INODE * fs_inode)
+ntfs_dinode_copy(NTFS_INFO * ntfs, FS_INODE * fs_inode)
 {
     ntfs_mft *mft = ntfs->mft;
     ntfs_attr *attr;
@@ -1253,6 +1263,8 @@ ntfs_copy_inode(NTFS_INFO * ntfs, FS_INODE * fs_inode)
     fs_inode->nlink = getu16(fs, mft->link);
     fs_inode->seq = getu16(fs, mft->seq);
 
+    fs_inode->addr = ntfs->mnum;
+
     /* Set the mode for file or directory */
     if (getu16(fs, ntfs->mft->flags) & NTFS_MFT_DIR)
 	fs_inode->mode = FS_INODE_DIR;
@@ -1298,10 +1310,10 @@ ntfs_copy_inode(NTFS_INFO * ntfs, FS_INODE * fs_inode)
  * Also sets the ntfs->mnum value
  */
 void
-ntfs_dinode_lookup(NTFS_INFO * ntfs, INUM_T mftnum)
+ntfs_dinode_load(NTFS_INFO * ntfs, INUM_T mftnum)
 {
     /* mft_lookup does a sanity check, so we can skip it here */
-    ntfs_mft_lookup(ntfs, ntfs->mft, mftnum);
+    ntfs_dinode_lookup(ntfs, ntfs->mft, mftnum);
     ntfs->mnum = mftnum;
 
     return;
@@ -1318,10 +1330,10 @@ ntfs_inode_lookup(FS_INFO * fs, INUM_T mftnum)
     FS_INODE *fs_inode = fs_inode_alloc(NTFS_NDADDR, NTFS_NIADDR);
 
     /* Lookup inode and store it in the ntfs structure */
-    ntfs_dinode_lookup(ntfs, mftnum);
+    ntfs_dinode_load(ntfs, mftnum);
 
     /* Copy the structure in ntfs to generic fs_inode */
-    ntfs_copy_inode(ntfs, fs_inode);
+    ntfs_dinode_copy(ntfs, fs_inode);
 
     return (fs_inode);
 }
@@ -1364,7 +1376,7 @@ ntfs_load_attrdef(NTFS_INFO * ntfs)
     load_file.base = load_file.cur = mymalloc(fs_data->size);
     ntfs->attrdef = (ntfs_attrdef *) load_file.base;
 
-    ntfs_data_walk(ntfs, fs_data, 0, load_file_action,
+    ntfs_data_walk(ntfs, fs_inode->addr, fs_data, 0, load_file_action,
 		   (void *) &load_file);
 
     if (load_file.left > 0)
@@ -1410,7 +1422,7 @@ ntfs_load_bmap(NTFS_INFO * ntfs)
     FS_INFO *fs = &ntfs->fs_info;
 
     /* Get data on the bitmap */
-    ntfs_dinode_lookup(ntfs, NTFS_MFT_BMAP);
+    ntfs_dinode_load(ntfs, NTFS_MFT_BMAP);
 
     attr = (ntfs_attr *) ((uintptr_t) ntfs->mft +
 			  getu16(fs, ntfs->mft->attr_off));
@@ -1451,7 +1463,6 @@ ntfs_load_bmap(NTFS_INFO * ntfs)
 	error("ntfs_load_bmap: Error reading block at %" PRIuDADDR ": %m",
 	      ntfs->bmap->addr);
     }
-
 }
 
 
@@ -1551,7 +1562,7 @@ ntfs_file_walk(FS_INFO * fs, FS_INODE * fs_inode, uint32_t type,
 
 
     /* process the content */
-    ntfs_data_walk(ntfs, fs_data, flags, action, ptr);
+    ntfs_data_walk(ntfs, fs_inode->addr, fs_data, flags, action, ptr);
 
     return;
 }
@@ -1653,7 +1664,7 @@ ntfs_inode_walk(FS_INFO * fs, INUM_T start_inum, INUM_T end_inum,
     for (mftnum = start_inum; mftnum <= end_inum; mftnum++) {
 
 	/* read MFT entry in to NTFS_INFO */
-	ntfs_dinode_lookup(ntfs, mftnum);
+	ntfs_dinode_load(ntfs, mftnum);
 
 	/* we only want to look at base file records 
 	 * (extended are because the base could not fit into one)
@@ -1680,12 +1691,11 @@ ntfs_inode_walk(FS_INFO * fs, INUM_T start_inum, INUM_T end_inum,
 	if ((flags & myflags) != myflags)
 	    continue;
 
-
 	/* copy into generic format */
-	ntfs_copy_inode(ntfs, fs_inode);
+	ntfs_dinode_copy(ntfs, fs_inode);
 
 	/* call action */
-	if (WALK_STOP == action(fs, mftnum, fs_inode, myflags, ptr)) {
+	if (WALK_STOP == action(fs, fs_inode, myflags, ptr)) {
 	    fs_inode_free(fs_inode);
 	    return;
 	}
@@ -1841,7 +1851,7 @@ typedef struct {
 
 static uint8_t
 print_addr_act(FS_INFO * fs, DADDR_T addr, char *buf,
-	       int size, int flags, void *ptr)
+	       unsigned int size, int flags, void *ptr)
 {
     NTFS_PRINT_ADDR *print = (NTFS_PRINT_ADDR *) ptr;
     fprintf(print->hFile, "%" PRIuDADDR " ", addr);
@@ -2176,7 +2186,7 @@ ntfs_istat(FS_INFO * fs, FILE * hFile, INUM_T inum, int numblock,
 	load_file.cur = load_file.base = buf = mymalloc(fs_data->size);
 	endaddr = (uintptr_t) buf + fs_data->size;
 
-	ntfs_data_walk(ntfs, fs_data, 0, load_file_action,
+	ntfs_data_walk(ntfs, fs_inode->addr, fs_data, 0, load_file_action,
 		       (void *) &load_file);
 
 	/* this value should be zero, if not then we didn't read all of the
@@ -2299,9 +2309,9 @@ ntfs_open(IMG_INFO * img_info, unsigned char ftype, uint8_t test)
     char *myname = "ntfs_open";
     NTFS_INFO *ntfs = (NTFS_INFO *) mymalloc(sizeof(*ntfs));
     FS_INFO *fs = &(ntfs->fs_info);
-    int len;
+    unsigned int len;
 
-    loading_the_MFT = 0;
+    ntfs->loading_the_MFT = 0;
     ntfs->bmap = NULL;
     ntfs->bmap_buf = NULL;
 
@@ -2449,11 +2459,11 @@ ntfs_open(IMG_INFO * img_info, unsigned char ftype, uint8_t test)
 
 
     /* load the data run for the MFT table into ntfs->mft */
-    loading_the_MFT = 1;
-    ntfs_mft_lookup(ntfs, ntfs->mft, NTFS_MFT_MFT);
+    ntfs->loading_the_MFT = 1;
+    ntfs_dinode_lookup(ntfs, ntfs->mft, NTFS_MFT_MFT);
 
     ntfs->mft_inode = fs_inode_alloc(NTFS_NDADDR, NTFS_NIADDR);
-    ntfs_copy_inode(ntfs, ntfs->mft_inode);
+    ntfs_dinode_copy(ntfs, ntfs->mft_inode);
 
     /* cache the data attribute 
      *
@@ -2478,19 +2488,17 @@ ntfs_open(IMG_INFO * img_info, unsigned char ftype, uint8_t test)
     fs->last_inum = fs->inum_count - 1;
 
     /* reset the flag that we are no longer loading $MFT */
-    loading_the_MFT = 0;
+    ntfs->loading_the_MFT = 0;
 
     /* load the version of the file system */
     ntfs_load_ver(ntfs);
 
-
     /* load the data block bitmap data run into ntfs_info */
     ntfs_load_bmap(ntfs);
 
     /* set this to NULL and it will be loaded if needed */
     ntfs->attrdef = NULL;
 
-
     fs->jblk_walk = ntfs_jblk_walk;
     fs->jentry_walk = ntfs_jentry_walk;
     fs->jopen = ntfs_jopen;
