@@ -1,7 +1,7 @@
 /*
 ** The Sleuth Kit
 **
-** $Date: 2005/01/17 22:40:16 $
+** $Date: 2005/07/08 17:20:10 $
 **
 ** Brian Carrier [carrier@sleuthkit.org]
 ** Copyright (c) 2003-2005 Brian Carrier.  All rights reserved
@@ -335,10 +335,10 @@ typedef struct {
 } ntfs_runlist;
 
 #define NTFS_RUNL_LENSZ(runl)	\
-	(runl->len & 0x0f)
+	(uint8_t)(runl->len & 0x0f)
 
 #define NTFS_RUNL_OFFSZ(runl)	\
-	((runl->len & 0xf0) >> 4)
+	(uint8_t)((runl->len & 0xf0) >> 4)
 
 
 /************************************************************************
@@ -478,6 +478,8 @@ typedef struct {
     uint16_t idx_rsize_b;	/* number of bytes per idx record */
     DADDR_T root_mft_addr;	/* address of first mft entry */
 
+    uint8_t loading_the_MFT;	/* set to 1 when initializing the setup */
+
     FS_DATA_RUN *bmap;		/* Run of bitmap for clusters (linked list) */
     DATA_BUF *bmap_buf;		/* buffer to hold cached copy of bitmap */
     DADDR_T bmap_buf_off;	/* offset cluster in cached bitmap */
@@ -486,7 +488,7 @@ typedef struct {
 } NTFS_INFO;
 
 extern void
- ntfs_data_walk(NTFS_INFO *, FS_DATA *, int, FS_FILE_WALK_FN, void *);
+ ntfs_data_walk(NTFS_INFO *, INUM_T, FS_DATA *, int, FS_FILE_WALK_FN, void *);
 
 extern void
  ntfs_dent_walk(FS_INFO *, INUM_T, int, FS_DENT_WALK_FN, void *);
