@@ -2,7 +2,7 @@
 ** ntfs_dent
 ** The Sleuth Kit
 **
-** $Date: 2005/01/17 22:40:16 $
+** $Date: 2005/06/15 15:58:03 $
 **
 ** name layer support for the NTFS file system
 **
@@ -19,22 +19,35 @@
 
 
 
-/* recursive path stuff */
-static unsigned int depth = 0;	/* how deep in the directory tree are we */
 #define MAX_DEPTH   64
-static char *didx[MAX_DEPTH];	/* pointer in dirs string to where '/' is for
-				 ** given depth */
 #define DIR_STRSZ  2048
-static char dirs[DIR_STRSZ];	/* The current directory name string */
+
+typedef struct {
+    /* Recursive path stuff */
+
+    /* how deep in the directory tree are we */
+    unsigned int depth;
+
+    /* pointer in dirs string to where '/' is for given depth */
+    char *didx[MAX_DEPTH];
+
+    /* The current directory name string */
+    char dirs[DIR_STRSZ];
+
+} NTFS_DINFO;
 
 
+static void
+ ntfs_dent_walk_lcl(FS_INFO *, NTFS_DINFO *, INUM_T, int,
+		    FS_DENT_WALK_FN, void *);
 /* 
  * copy the index (directory) entry into the generic structure
  *
  * uses the global variables 'dirs' and 'depth'
  */
 static void
-ntfs_dent_copy(NTFS_INFO * ntfs, ntfs_idxentry * idxe, FS_DENT * fs_dent)
+ntfs_dent_copy(NTFS_INFO * ntfs, NTFS_DINFO * dinfo, ntfs_idxentry * idxe,
+	       FS_DENT * fs_dent)
 {
     ntfs_attr_fname *fname = (ntfs_attr_fname *) & idxe->stream;
     FS_INFO *fs = (FS_INFO *) & ntfs->fs_info;
@@ -47,8 +60,8 @@ ntfs_dent_copy(NTFS_INFO * ntfs, ntfs_idxentry * idxe, FS_DENT * fs_dent)
 	      fs_dent->name, fs_dent->maxnamlen);
 
     /* copy the path data */
-    fs_dent->path = dirs;
-    fs_dent->pathdepth = depth;
+    fs_dent->path = dinfo->dirs;
+    fs_dent->pathdepth = dinfo->depth;
 
     /* Get the actual inode */
     if (fs_dent->fsi != NULL)
@@ -99,11 +112,13 @@ is_time(uint64_t t)
  *
  * The only flag that we care about is ALLOC (no UNALLOC entries exist
  * in the tree)
+ *
+ * Return WALK_STOP if the action returned stop -- otherwise return WALK_CONT
  */
-static void
-ntfs_dent_idxentry(NTFS_INFO * ntfs, ntfs_idxentry * idxe,
-		   uint32_t size, uint32_t len, int flags,
-		   FS_DENT_WALK_FN action, void *ptr)
+static int
+ntfs_dent_idxentry(NTFS_INFO * ntfs, NTFS_DINFO * dinfo,
+		   ntfs_idxentry * idxe, uint32_t size, uint32_t len,
+		   int flags, FS_DENT_WALK_FN action, void *ptr)
 {
     uintptr_t endaddr, endaddr_alloc;
     int myflags = 0;
@@ -208,7 +223,7 @@ ntfs_dent_idxentry(NTFS_INFO * ntfs, ntfs_idxentry * idxe,
 
 
 	/* Copy it into the generic form */
-	ntfs_dent_copy(ntfs, idxe, fs_dent);
+	ntfs_dent_copy(ntfs, dinfo, idxe, fs_dent);
 
 	if (verbose)
 	    fprintf(logfp,
@@ -244,7 +259,7 @@ ntfs_dent_idxentry(NTFS_INFO * ntfs, ntfs_idxentry * idxe,
 
 	if ((flags & myflags) == myflags) {
 	    if (WALK_STOP == action(fs, fs_dent, myflags, ptr)) {
-		return;
+		return WALK_STOP;
 	    }
 	}
 
@@ -255,20 +270,21 @@ ntfs_dent_idxentry(NTFS_INFO * ntfs, ntfs_idxentry * idxe,
 	    ((fs_dent->fsi->mode & FS_INODE_FMT) == FS_INODE_DIR) &&
 	    (fs_dent->inode)) {
 
-	    if (depth < MAX_DEPTH) {
-		didx[depth] = &dirs[strlen(dirs)];
-		strncpy(didx[depth], fs_dent->name,
-			DIR_STRSZ - strlen(dirs));
-		strncat(dirs, "/", DIR_STRSZ);
+	    if (dinfo->depth < MAX_DEPTH) {
+		dinfo->didx[dinfo->depth] =
+		    &dinfo->dirs[strlen(dinfo->dirs)];
+		strncpy(dinfo->didx[dinfo->depth], fs_dent->name,
+			DIR_STRSZ - strlen(dinfo->dirs));
+		strncat(dinfo->dirs, "/", DIR_STRSZ);
 	    }
-	    depth++;
+	    dinfo->depth++;
 
-	    ntfs_dent_walk(&(ntfs->fs_info), fs_dent->inode, flags,
-			   action, ptr);
+	    ntfs_dent_walk_lcl(&(ntfs->fs_info), dinfo, fs_dent->inode,
+			       flags, action, ptr);
 
-	    depth--;
-	    if (depth < MAX_DEPTH)
-		*didx[depth] = '\0';
+	    dinfo->depth--;
+	    if (dinfo->depth < MAX_DEPTH)
+		*dinfo->didx[dinfo->depth] = '\0';
 
 	}			/* end of recurse */
 
@@ -299,7 +315,7 @@ ntfs_dent_idxentry(NTFS_INFO * ntfs, ntfs_idxentry * idxe,
 
     fs_dent_free(fs_dent);
 
-    return;
+    return WALK_CONT;
 
 }
 
@@ -325,7 +341,8 @@ ntfs_fix_idxrec(NTFS_INFO * ntfs, ntfs_idxrec * idxrec, uint32_t len)
 		PRIu32 "\n", (uint64_t) ((uintptr_t) idxrec), len);
 
     /* sanity check so we don't run over in the next loop */
-    if ((getu16(fs, idxrec->upd_cnt) - 1) * ntfs->ssize_b > len)
+    if ((unsigned int) ((getu16(fs, idxrec->upd_cnt) - 1) *
+			ntfs->ssize_b) > len)
 	error("More Update Sequence Entries than idx record size");
 
     /* Apply the update sequence structure template */
@@ -339,7 +356,7 @@ ntfs_fix_idxrec(NTFS_INFO * ntfs, ntfs_idxrec * idxrec, uint32_t len)
 
 	/* The offset into the buffer of the value to analyze */
 	int offset = i * ntfs->ssize_b - 2;
-	uint8_t *new, *old;
+	uint8_t *new_val, *old_val;
 
 	/* get the current sequence value */
 	uint16_t cur_seq = getu16(fs, (uintptr_t) idxrec + offset);
@@ -356,17 +373,17 @@ ntfs_fix_idxrec(NTFS_INFO * ntfs, ntfs_idxrec * idxrec, uint32_t len)
 		 orig_seq, cur_seq, cur_repl);
 	}
 
-	new = &upd->upd_seq + (i - 1) * 2;
-	old = (uint8_t *) ((uintptr_t) idxrec + offset);
+	new_val = &upd->upd_seq + (i - 1) * 2;
+	old_val = (uint8_t *) ((uintptr_t) idxrec + offset);
 
 	if (verbose)
 	    fprintf(logfp,
 		    "ntfs_fix_idxrec: upd_seq %i   Replacing: %.4" PRIx16
-		    "   With: %.4" PRIx16 "\n", i, getu16(fs, old),
-		    getu16(fs, new));
+		    "   With: %.4" PRIx16 "\n", i, getu16(fs, old_val),
+		    getu16(fs, new_val));
 
-	*old++ = *new++;
-	*old = *new;
+	*old_val++ = *new_val++;
+	*old_val = *new_val;
     }
 
     return;
@@ -384,6 +401,19 @@ void
 ntfs_dent_walk(FS_INFO * fs, INUM_T inum, int flags,
 	       FS_DENT_WALK_FN action, void *ptr)
 {
+    NTFS_DINFO dinfo;
+
+    memset(&dinfo, 0, sizeof(NTFS_DINFO));
+
+    ntfs_dent_walk_lcl(fs, &dinfo, inum, flags, action, ptr);
+}
+
+
+static void
+ntfs_dent_walk_lcl(FS_INFO * fs, NTFS_DINFO * dinfo, INUM_T inum,
+		   int flags, FS_DENT_WALK_FN action, void *ptr)
+{
+
     NTFS_INFO *ntfs = (NTFS_INFO *) fs;
     FS_INODE *fs_inode;
     FS_DATA *fs_data_root, *fs_data_alloc;
@@ -433,8 +463,8 @@ ntfs_dent_walk(FS_INFO * fs, INUM_T inum, int flags,
 	strcpy(fs_dent->name, ".");
 
 	/* copy the path data */
-	fs_dent->path = dirs;
-	fs_dent->pathdepth = depth;
+	fs_dent->path = dinfo->dirs;
+	fs_dent->pathdepth = dinfo->depth;
 
 	/* this is probably a waste, but just in case the action mucks
 	 * with it ...
@@ -511,14 +541,17 @@ ntfs_dent_walk(FS_INFO * fs, INUM_T inum, int flags,
 			      getu32(fs, idxelist->begin_off));
 
     /* Process $IDX_ROOT  */
-    ntfs_dent_idxentry(ntfs, idxe,
-		       getu32(fs, idxelist->buf_off) - getu32(fs,
-							      idxelist->
-							      begin_off),
-		       getu32(fs, idxelist->end_off) - getu32(fs,
-							      idxelist->
-							      begin_off),
-		       flags, action, ptr);
+    if (WALK_STOP == ntfs_dent_idxentry(ntfs, dinfo, idxe,
+					getu32(fs,
+					       idxelist->buf_off) -
+					getu32(fs, idxelist->begin_off),
+					getu32(fs,
+					       idxelist->end_off) -
+					getu32(fs, idxelist->begin_off),
+					flags, action, ptr)) {
+	fs_inode_free(fs_inode);
+	return;
+    }
 
     /* 
      * get the index allocation attribute if it exists (it doesn't for 
@@ -557,7 +590,7 @@ ntfs_dent_walk(FS_INFO * fs, INUM_T inum, int flags,
     if (verbose)
 	fprintf(logfp, "ntfs_dent_walk: Copying $IDX_ALLOC into buffer\n");
 
-    ntfs_data_walk(ntfs, fs_data_alloc,
+    ntfs_data_walk(ntfs, fs_inode->addr, fs_data_alloc,
 		   FS_FLAG_FILE_SLACK, load_file_action,
 		   (void *) &load_file);
 
@@ -634,11 +667,16 @@ ntfs_dent_walk(FS_INFO * fs, INUM_T inum, int flags,
 	list_len = (uintptr_t) idxrec - (uintptr_t) idxe;
 
 	/* process the list of index entries */
-	ntfs_dent_idxentry(ntfs, idxe, list_len,
-			   getu32(fs, idxelist->end_off) - getu32(fs,
-								  idxelist->
-								  begin_off),
-			   flags, action, ptr);
+	if (WALK_STOP == ntfs_dent_idxentry(ntfs, dinfo, idxe, list_len,
+					    getu32(fs,
+						   idxelist->end_off) -
+					    getu32(fs,
+						   idxelist->begin_off),
+					    flags, action, ptr)) {
+	    fs_inode_free(fs_inode);
+	    free(idxalloc);
+	    return;
+	}
 
 	/* reset the pointer to the next record */
 	idxrec_p = idxrec;
@@ -671,18 +709,22 @@ ntfs_dent_walk(FS_INFO * fs, INUM_T inum, int flags,
 	    ((uintptr_t) idxalloc + idxalloc_len) - (uintptr_t) idxe;
 
 	/* process the list of index entries */
-	ntfs_dent_idxentry(ntfs, idxe, list_len,
-			   getu32(fs, idxelist->end_off) - getu32(fs,
-								  idxelist->
-								  begin_off),
-			   flags, action, ptr);
+	if (WALK_STOP == ntfs_dent_idxentry(ntfs, dinfo, idxe, list_len,
+					    getu32(fs,
+						   idxelist->end_off) -
+					    getu32(fs,
+						   idxelist->begin_off),
+					    flags, action, ptr)) {
+	    fs_inode_free(fs_inode);
+	    free(idxalloc);
+	    return;
+	}
     }
 
     fs_inode_free(fs_inode);
     free(idxalloc);
 
     return;
-
 }				/* end of dent_walk */
 
 
@@ -702,9 +744,9 @@ ntfs_dent_walk(FS_INFO * fs, INUM_T inum, int flags,
  * added to it before action is called
  */
 static void
-ntfs_find_file_rec(FS_INFO * fs, FS_DENT * fs_dent, FS_NAME * fs_name,
-		   int myflags, int flags, FS_DENT_WALK_FN action,
-		   void *ptr)
+ntfs_find_file_rec(FS_INFO * fs, NTFS_DINFO * dinfo, FS_DENT * fs_dent,
+		   FS_NAME * fs_name, int myflags, int flags,
+		   FS_DENT_WALK_FN action, void *ptr)
 {
     FS_INODE *fs_inode_par;
     FS_NAME *fs_name_par;
@@ -732,12 +774,13 @@ ntfs_find_file_rec(FS_INFO * fs, FS_DENT * fs_dent, FS_NAME * fs_name,
 	/* @@@ There should be a sanity check here to verify that the 
 	 * previous name was unallocated ... but how do I get it again?
 	 */
-	if ((((uintptr_t) didx[depth - 1] - len) >= (uintptr_t) & dirs[0])
-	    && (depth < MAX_DEPTH)) {
-	    begin = didx[depth] =
-		(char *) ((uintptr_t) didx[depth - 1] - len);
+	if ((((uintptr_t) dinfo->didx[dinfo->depth - 1] - len) >=
+	     (uintptr_t) & dinfo->dirs[0])
+	    && (dinfo->depth < MAX_DEPTH)) {
+	    begin = dinfo->didx[dinfo->depth] =
+		(char *) ((uintptr_t) dinfo->didx[dinfo->depth - 1] - len);
 
-	    depth++;
+	    dinfo->depth++;
 	    decrem = 1;
 
 	    for (i = 0; i < len; i++)
@@ -745,11 +788,11 @@ ntfs_find_file_rec(FS_INFO * fs, FS_DENT * fs_dent, FS_NAME * fs_name,
 	}
 
 	fs_dent->path = begin;
-	fs_dent->pathdepth = depth;
+	fs_dent->pathdepth = dinfo->depth;
 	action(fs, fs_dent, myflags, ptr);
 
 	if (decrem)
-	    depth--;
+	    dinfo->depth--;
 
 	fs_inode_free(fs_inode_par);
 	return;
@@ -762,12 +805,13 @@ ntfs_find_file_rec(FS_INFO * fs, FS_DENT * fs_dent, FS_NAME * fs_name,
 
 	/* do some length checks on the dir structure 
 	 * if we can't fit it then forget about it */
-	if ((((uintptr_t) didx[depth - 1] - len - 1) >=
-	     (uintptr_t) & dirs[0]) && (depth < MAX_DEPTH)) {
-	    begin = didx[depth] =
-		(char *) ((uintptr_t) didx[depth - 1] - len - 1);
+	if ((((uintptr_t) dinfo->didx[dinfo->depth - 1] - len - 1) >=
+	     (uintptr_t) & dinfo->dirs[0]) && (dinfo->depth < MAX_DEPTH)) {
+	    begin = dinfo->didx[dinfo->depth] =
+		(char *) ((uintptr_t) dinfo->didx[dinfo->depth - 1] - len -
+			  1);
 
-	    depth++;
+	    dinfo->depth++;
 	    decrem = 1;
 
 	    *begin = '/';
@@ -775,7 +819,7 @@ ntfs_find_file_rec(FS_INFO * fs, FS_DENT * fs_dent, FS_NAME * fs_name,
 		begin[i + 1] = fs_name_par->name[i];
 	}
 	else {
-	    begin = didx[depth];
+	    begin = dinfo->didx[dinfo->depth];
 	    decrem = 0;
 	}
 
@@ -789,19 +833,19 @@ ntfs_find_file_rec(FS_INFO * fs, FS_DENT * fs_dent, FS_NAME * fs_name,
 	     * the beginning
 	     */
 	    fs_dent->path = (char *) ((uintptr_t) begin + 1);
-	    fs_dent->pathdepth = depth;
+	    fs_dent->pathdepth = dinfo->depth;
 	    action(fs, fs_dent, myflags, ptr);
 	}
 
 	/* otherwise, recurse some more */
 	else {
-	    ntfs_find_file_rec(fs, fs_dent, fs_name_par, myflags, flags,
-			       action, ptr);
+	    ntfs_find_file_rec(fs, dinfo, fs_dent, fs_name_par, myflags,
+			       flags, action, ptr);
 	}
 
 	/* if we incremented before, then decrement the depth now */
 	if (decrem)
-	    depth--;
+	    dinfo->depth--;
     }
     fs_inode_free(fs_inode_par);
 }
@@ -828,22 +872,26 @@ ntfs_find_file(FS_INFO * fs, INUM_T inode_toid, uint32_t type_toid,
     int myflags;
     NTFS_INFO *ntfs = (NTFS_INFO *) fs;
     char *attr = NULL;
+    NTFS_DINFO dinfo;
+
 
     /* sanity check */
     if (inode_toid < fs->first_inum || inode_toid > fs->last_inum)
 	error("invalid inode value: " PRIuINUM "i\n", inode_toid);
 
-    /* in this function, we use the dirs array in the opposite order.
+    memset(&dinfo, 0, sizeof(NTFS_DINFO));
+
+    /* in this function, we use the dinfo->dirs array in the opposite order.
      * we set the end of it to NULL and then prepend the
      * directories to it
      *
-     * didx[depth] will point to where the current level started their
+     * dinfo->didx[dinfo->depth] will point to where the current level started their
      * dir name
      */
-    dirs[DIR_STRSZ - 2] = '/';
-    dirs[DIR_STRSZ - 1] = '\0';
-    didx[0] = &dirs[DIR_STRSZ - 2];
-    depth = 1;
+    dinfo.dirs[DIR_STRSZ - 2] = '/';
+    dinfo.dirs[DIR_STRSZ - 1] = '\0';
+    dinfo.didx[0] = &dinfo.dirs[DIR_STRSZ - 2];
+    dinfo.depth = 1;
 
 
     /* lookup the inode and get its allocation status */
@@ -883,14 +931,14 @@ ntfs_find_file(FS_INFO * fs, INUM_T inode_toid, uint32_t type_toid,
 
 	/* if this is in the root directory, then call back */
 	if (fs_name->par_inode == NTFS_ROOTINO) {
-	    fs_dent->path = didx[0];
-	    fs_dent->pathdepth = depth;
+	    fs_dent->path = dinfo.didx[0];
+	    fs_dent->pathdepth = dinfo.depth;
 	    action(fs, fs_dent, myflags, ptr);
 	}
 	/* call the recursive function on the parent */
 	else {
-	    ntfs_find_file_rec(fs, fs_dent, fs_name, myflags, flags,
-			       action, ptr);
+	    ntfs_find_file_rec(fs, &dinfo, fs_dent, fs_name, myflags,
+			       flags, action, ptr);
 	}
 
     }				/* end of name loop */
