@@ -1,5 +1,5 @@
 /*
- * $Date: 2005/04/08 18:01:21 $
+ * $Date: 2005/06/13 19:27:22 $
  *
  * Brian Carrier [carrier@sleuthkit.org]
  * Copyright (c) 2005 Brian Carrier.  All rights reserved
@@ -60,7 +60,8 @@ split_read_random(IMG_INFO * img_info, char *buf, OFF_T len, OFF_T offset)
 
 	/* Does the data start in this image? */
 	if (tot_offset < split_info->max_off[i]) {
-	    OFF_T rel_offset, cnt, read_len;
+	    off_t rel_offset;
+	    OFF_T cnt, read_len;
 	    IMG_SPLIT_CACHE *cimg;;
 
 
