@@ -62,7 +62,7 @@ mymalloc(int len)
 {
     char *ptr;
 
-    if ((ptr = malloc(len)) == 0)
+    if ((ptr = (char *) malloc(len)) == 0)
 	error("Insufficient memory: %m");
     return (ptr);
 }
@@ -72,7 +72,7 @@ mymalloc(int len)
 char *
 myrealloc(char *ptr, int len)
 {
-    if ((ptr = realloc(ptr, len)) == 0)
+    if ((ptr = (char *) realloc(ptr, len)) == 0)
 	error("Insufficient memory: %m");
     return (ptr);
 }
