@@ -1,7 +1,7 @@
 /*
  * The Sleuth Kit
  *
- * $Date: 2005/01/17 22:40:17 $
+ * $Date: 2005/04/01 18:56:49 $
  *
  * Brian Carrier [carrier@sleuthkit.org]
  * Copyright (c) 2003-2005 Brian Carrier.  All rights reserved
@@ -869,19 +869,6 @@ dos_load_prim_table(MM_INFO * mm, uint8_t test)
 			"dos_load_prim_table: Test fail - NTFS OEM name\n");
 	    return 1;
 	}
-
-
-	if (((getu16(mm, &sect.ssize) % 512) == 0)
-	    && ((sect.csize == 0x01) || (sect.csize == 0x02)
-		|| (sect.csize == 0x04) || (sect.csize == 0x08)
-		|| (sect.csize == 0x10) || (sect.csize == 0x20)
-		|| (sect.csize == 0x40) || (sect.csize == 0x80))) {
-	    if (verbose)
-		fprintf(logfp,
-			"dos_load_prim_table: Test fail - Valid sector & cluster size\n");
-	    return 1;
-	}
-
     }
 
     /* Add an entry of 1 sector for the table  to the internal structure */
