@@ -1,7 +1,7 @@
 /*
  * The Sleuth Kit
  *
- * $Date: 2005/01/17 22:40:17 $
+ * $Date: 2005/06/13 19:27:25 $
  *
  * Brian Carrier [carrier@sleuthkit.org]
  * Copyright (c) 2004-2005 Brian Carrier.  All rights reserved
@@ -53,7 +53,7 @@ gpt_load_table(MM_INFO * mm, uint8_t test)
     gpt_head head;
     gpt_entry *ent;
     dos_sect dos_part;
-    int i, a;
+    unsigned int i, a;
     uint32_t ent_size;
     char *safe_str, *head_str, *tab_str, *ent_buf;
     if (verbose)
@@ -189,7 +189,7 @@ gpt_load_table(MM_INFO * mm, uint8_t test)
 
 
 	    name = mymalloc(72);
-	    uni2ascii(ent->name, 72, (uint8_t *) name, 72);
+	    uni2ascii((char *)ent->name, 72, name, 72);
 	    mm_part_add(mm, (DADDR_T) getu64(mm, ent->start_lba),
 			(DADDR_T) (getu64(mm, ent->end_lba) -
 				   getu64(mm, ent->start_lba) + 1),
@@ -210,7 +210,7 @@ gpt_part_walk(MM_INFO * mm, PNUM_T start, PNUM_T last, int flags,
 	      MM_PART_WALK_FN action, char *ptr)
 {
     MM_PART *part;
-    int cnt = 0;
+    unsigned int cnt = 0;
 
     if (start < mm->first_part || start > mm->last_part)
 	error("Invalid starting partition: %" PRIuPNUM "", start);
