@@ -1,7 +1,7 @@
 /*
  * The Sleuth Kit
  *
- * $Date: 2005/01/17 22:40:17 $
+ * $Date: 2005/06/13 19:27:25 $
  *
  * Brian Carrier [carrier@sleuthkit.org]
  * Copyright (c) 2003-2005 Brian Carrier.  All rights reserved
@@ -66,8 +66,8 @@ struct MM_INFO {
     uint8_t mmtype;		/* type of media management */
     DADDR_T sect_offset;
     char *str_type;
-    int block_size;
-    int dev_bsize;
+    unsigned int block_size;
+    unsigned int dev_bsize;
 
     /* endian ordering flag - values given in tsk_endian.h */
     uint8_t endian;
