@@ -1,7 +1,7 @@
 /*
  * The Sleuth Kit
  *
- * $Date: 2005/01/17 22:40:17 $
+ * $Date: 2005/06/13 19:27:25 $
  *
  * Brian Carrier [carrier@sleuthkit.org]
  * Copyright (c) 2003-2005 Brian Carrier.  All rights reserved
@@ -113,7 +113,7 @@ part_act(MM_INFO * mm, PNUM_T pnum, MM_PART * part, int flag, char *ptr)
 	    size = part->len / (2 << 10);
 	    unit = 'M';
 	}
-	else if (size < (2 << 30)) {
+	else if (size < ((OFF_T) 2 << 30)) {
 	    size = part->len / (2 << 20);
 	    unit = 'G';
 	}
