@@ -1,7 +1,7 @@
 /*
  * The Sleuth Kit
  *
- * $Date: 2005/01/17 22:40:17 $
+ * $Date: 2005/06/13 19:27:26 $
  *
  * Brian Carrier [carrier@sleuthkit.org]
  * Copyright (c) 2003-2005 Brian Carrier.  All rights reserved
@@ -301,7 +301,7 @@ sun_part_walk(MM_INFO * mm, PNUM_T start, PNUM_T last, int flags,
 	      MM_PART_WALK_FN action, char *ptr)
 {
     MM_PART *part;
-    int cnt = 0;
+    unsigned int cnt = 0;
 
     if (start < mm->first_part || start > mm->last_part)
 	error("Invalid starting partition: %d", start);
